@@ -36,6 +36,7 @@ import java.util.Collection;
 import java.util.logging.Logger;
 
 import org.acegisecurity.Authentication;
+import org.acegisecurity.context.SecurityContext;
 import org.acegisecurity.context.SecurityContextHolder;
 
 /**
@@ -54,9 +55,7 @@ public class DependencyRunner implements Runnable {
     }
 
     public void run() {
-        Authentication saveAuth = SecurityContextHolder.getContext().getAuthentication();
-        SecurityContextHolder.getContext().setAuthentication(ACL.SYSTEM);
-
+        SecurityContext oldContext = ACL.impersonate(ACL.SYSTEM);
         try {
             Set<AbstractProject> topLevelProjects = new HashSet<AbstractProject>();
             // Get all top-level projects
@@ -74,7 +73,7 @@ public class DependencyRunner implements Runnable {
                 runnable.run(p);
             }
         } finally {
-            SecurityContextHolder.getContext().setAuthentication(saveAuth);
+            SecurityContextHolder.setContext(oldContext);
         }
     }
 
