@@ -47,6 +47,7 @@ import hudson.tasks.BuildWrapper;
 import hudson.tasks.BuildWrappers;
 import hudson.tasks.Builder;
 import hudson.tasks.Publisher;
+import hudson.tasks.UserAvatarResolver;
 import hudson.util.Area;
 import hudson.util.Iterators;
 import hudson.scm.SCM;
@@ -55,6 +56,7 @@ import hudson.util.Secret;
 import hudson.views.MyViewsTabBar;
 import hudson.views.ViewsTabBar;
 import hudson.widgets.RenderOnDemandClosure;
+import jenkins.model.GlobalConfiguration;
 import jenkins.model.Jenkins;
 import org.acegisecurity.providers.anonymous.AnonymousAuthenticationToken;
 import org.apache.commons.jelly.JellyContext;
@@ -63,7 +65,7 @@ import org.apache.commons.jelly.Script;
 import org.apache.commons.jelly.XMLOutput;
 import org.apache.commons.jexl.parser.ASTSizeFunction;
 import org.apache.commons.jexl.util.Introspector;
-import org.jvnet.animal_sniffer.IgnoreJRERequirement;
+import org.codehaus.mojo.animal_sniffer.IgnoreJRERequirement;
 import org.jvnet.tiger_types.Types;
 import org.kohsuke.stapler.Ancestor;
 import org.kohsuke.stapler.Stapler;
@@ -71,6 +73,7 @@ import org.kohsuke.stapler.StaplerRequest;
 import org.kohsuke.stapler.StaplerResponse;
 import org.kohsuke.stapler.jelly.InternationalizedStringExpression.RawHtmlArgument;
 
+import javax.management.modelmbean.DescriptorSupport;
 import javax.servlet.ServletException;
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletRequest;
@@ -149,6 +152,16 @@ public class Functions {
     public static String rfc822Date(Calendar cal) {
         return Util.RFC822_DATETIME_FORMATTER.format(cal.getTime());
     }
+    
+    public static void initPageVariables(JellyContext context) {
+        String rootURL = Stapler.getCurrentRequest().getContextPath();
+        Functions h = new Functions();
+
+        context.setVariable("rootURL", rootURL);
+        context.setVariable("h", h);
+        context.setVariable("resURL",rootURL+getResourcePath());
+        context.setVariable("imagesURL",rootURL+getResourcePath()+"/images");
+    }
 
     /**
      * Given {@code c=MyList (extends ArrayList<Foo>), base=List}, compute the parameterization of 'base'
@@ -644,7 +657,7 @@ public class Functions {
         StringBuilder buf = new StringBuilder();
         buf.append(req.getScheme()).append("://");
         buf.append(req.getServerName());
-        if(req.getLocalPort()!=80)
+        if(! (req.getScheme().equals("http") && req.getLocalPort()==80 || req.getScheme().equals("https") && req.getLocalPort()==443))
             buf.append(':').append(req.getLocalPort());
         buf.append(req.getContextPath()).append('/');
         return buf.toString();
@@ -730,21 +743,58 @@ public class Functions {
     /**
      * Gets all the descriptors sorted by their inheritance tree of {@link Describable}
      * so that descriptors of similar types come nearby.
+     *
+     * <p>
+     * We sort them by {@link Extension#ordinal()} but only for {@link GlobalConfiguration}s,
+     * as the value is normally used to compare similar kinds of extensions, and we needed
+     * {@link GlobalConfiguration}s to be able to position themselves in a layer above.
+     * This however creates some asymmetry between regular {@link Descriptor}s and {@link GlobalConfiguration}s.
+     * Perhaps it is better to introduce another annotation element? But then,
+     * extensions shouldn't normally concern themselves about ordering too much, and the only reason
+     * we needed this for {@link GlobalConfiguration}s are for backward compatibility.
      */
     public static Collection<Descriptor> getSortedDescriptorsForGlobalConfig() {
-        Map<String,Descriptor> r = new TreeMap<String, Descriptor>();
-        for (Descriptor<?> d : Jenkins.getInstance().getExtensionList(Descriptor.class)) {
-            if (d.getGlobalConfigPage()==null)  continue;
-            r.put(buildSuperclassHierarchy(d.clazz, new StringBuilder()).toString(),d);
+        class Tag implements Comparable<Tag> {
+            double ordinal;
+            String hierarchy;
+            Descriptor d;
+
+            Tag(double ordinal, Descriptor d) {
+                this.ordinal = ordinal;
+                this.d = d;
+                this.hierarchy = buildSuperclassHierarchy(d.clazz, new StringBuilder()).toString();
+            }
+
+            private StringBuilder buildSuperclassHierarchy(Class c, StringBuilder buf) {
+                Class sc = c.getSuperclass();
+                if (sc!=null)   buildSuperclassHierarchy(sc,buf).append(':');
+                return buf.append(c.getName());
+            }
+
+            public int compareTo(Tag that) {
+                int r = Double.compare(this.ordinal, that.ordinal);
+                if (r!=0)   return -r; // descending for ordinal
+                return this.hierarchy.compareTo(that.hierarchy);
+            }
         }
-        return r.values();
+
+        ExtensionList<Descriptor> exts = Jenkins.getInstance().getExtensionList(Descriptor.class);
+        List<Tag> r = new ArrayList<Tag>(exts.size());
+
+        for (ExtensionComponent<Descriptor> c : exts.getComponents()) {
+            Descriptor d = c.getInstance();
+            if (d.getGlobalConfigPage()==null)  continue;
+
+            r.add(new Tag(d instanceof GlobalConfiguration ? c.ordinal() : 0, d));
+        }
+        Collections.sort(r);
+
+        List<Descriptor> answer = new ArrayList<Descriptor>(r.size());
+        for (Tag d : r) answer.add(d.d);
+
+        return DescriptorVisibilityFilter.apply(Jenkins.getInstance(),answer);
     }
 
-    private static StringBuilder buildSuperclassHierarchy(Class c, StringBuilder buf) {
-        Class sc = c.getSuperclass();
-        if (sc!=null)   buildSuperclassHierarchy(sc,buf).append(':');
-        return buf.append(c.getName());
-    }
 
     /**
      * Computes the path to the icon of the given action
@@ -752,7 +802,8 @@ public class Functions {
      */
     public static String getIconFilePath(Action a) {
         String name = a.getIconFileName();
-        if(name.startsWith("/"))
+        if (name==null)     return null;
+        if (name.startsWith("/"))
             return name.substring(1);
         else
             return "images/24x24/"+name;
@@ -1122,6 +1173,24 @@ public class Functions {
             return base;
     }
 
+    /**
+     * Combine path components via '/' while handling leading/trailing '/' to avoid duplicates.
+     */
+    public static String joinPath(String... components) {
+        StringBuilder buf = new StringBuilder();
+        for (String s : components) {
+            if (s.length()==0)  continue;
+
+            if (buf.length()>0) {
+                if (buf.charAt(buf.length()-1)!='/')
+                    buf.append('/');
+                if (s.charAt(0)=='/')   s=s.substring(1);
+            }
+            buf.append(s);
+        }
+        return buf.toString();
+    }
+
     /**
      * Computes the hyperlink to actions, to handle the situation when the {@link Action#getUrlName()}
      * returns absolute URL.
@@ -1129,13 +1198,13 @@ public class Functions {
     public static String getActionUrl(String itUrl,Action action) {
         String urlName = action.getUrlName();
         if(urlName==null)   return null;    // to avoid NPE and fail to render the whole page
-        if(SCHEME.matcher(urlName).matches())
+        if(SCHEME.matcher(urlName).find())
             return urlName; // absolute URL
         if(urlName.startsWith("/"))
-            return Stapler.getCurrentRequest().getContextPath()+urlName;
+            return joinPath(Stapler.getCurrentRequest().getContextPath(),urlName);
         else
             // relative URL name
-            return Stapler.getCurrentRequest().getContextPath()+'/'+itUrl+urlName;
+            return joinPath(Stapler.getCurrentRequest().getContextPath()+'/'+itUrl,urlName);
     }
 
     /**
@@ -1323,7 +1392,7 @@ public class Functions {
         return DescriptorVisibilityFilter.apply(context,descriptors);
     }
     
-    private static final Pattern SCHEME = Pattern.compile("[a-z]+://.+");
+    private static final Pattern SCHEME = Pattern.compile("^([a-zA-Z][a-zA-Z0-9+.-]*):");
 
     /**
      * Returns true if we are running unit tests.
@@ -1407,4 +1476,22 @@ public class Functions {
         return all;
     }
 
+    /**
+     * Returns an avatar image URL for the specified user and preferred image size
+     * @param user the user
+     * @param avatarSize the preferred size of the avatar image
+     * @return a URL string
+     * @since 1.433
+     */
+    public static String getAvatar(User user, String avatarSize) {
+        return UserAvatarResolver.resolve(user, avatarSize);
+    }
+
+    /**
+     * @deprecated as of 1.451
+     *      Use {@link #getAvatar}
+     */
+    public String getUserAvatar(User user, String avatarSize) {
+        return getAvatar(user,avatarSize);
+    }
 }
