@@ -23,22 +23,27 @@
  */
 package hudson;
 
-import static hudson.init.InitMilestone.PLUGINS_PREPARED;
-import static hudson.init.InitMilestone.PLUGINS_STARTED;
-import static hudson.init.InitMilestone.PLUGINS_LISTED;
-
 import hudson.PluginWrapper.Dependency;
+import hudson.init.InitMilestone;
 import hudson.init.InitStrategy;
 import hudson.init.InitializerFinder;
 import hudson.model.AbstractModelObject;
+import hudson.model.AdministrativeMonitor;
+import hudson.model.Api;
+import hudson.model.Descriptor;
 import hudson.model.Failure;
-import jenkins.model.Jenkins;
 import hudson.model.UpdateCenter;
 import hudson.model.UpdateSite;
 import hudson.util.CyclicGraphDetector;
 import hudson.util.CyclicGraphDetector.CycleDetectedException;
+import hudson.util.IOException2;
 import hudson.util.PersistedList;
 import hudson.util.Service;
+import jenkins.ClassLoaderReflectionToolkit;
+import jenkins.InitReactorRunner;
+import jenkins.RestartRequiredException;
+import jenkins.YesNoMaybe;
+import jenkins.model.Jenkins;
 import org.apache.commons.fileupload.FileItem;
 import org.apache.commons.fileupload.disk.DiskFileItemFactory;
 import org.apache.commons.fileupload.servlet.ServletFileUpload;
@@ -47,6 +52,7 @@ import org.apache.commons.io.FilenameUtils;
 import org.apache.commons.logging.LogFactory;
 import org.jvnet.hudson.reactor.Executable;
 import org.jvnet.hudson.reactor.Reactor;
+import org.jvnet.hudson.reactor.ReactorException;
 import org.jvnet.hudson.reactor.TaskBuilder;
 import org.jvnet.hudson.reactor.TaskGraphBuilder;
 import org.kohsuke.stapler.HttpRedirect;
@@ -55,35 +61,42 @@ import org.kohsuke.stapler.HttpResponses;
 import org.kohsuke.stapler.QueryParameter;
 import org.kohsuke.stapler.StaplerRequest;
 import org.kohsuke.stapler.StaplerResponse;
+import org.kohsuke.stapler.export.Exported;
+import org.kohsuke.stapler.export.ExportedBean;
 
 import javax.servlet.ServletContext;
 import javax.servlet.ServletException;
 import java.io.File;
 import java.io.IOException;
 import java.lang.ref.WeakReference;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
 import java.net.URL;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Enumeration;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Hashtable;
 import java.util.List;
 import java.util.ListIterator;
-import java.util.Set;
 import java.util.Map;
-import java.util.HashMap;
+import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
+import static hudson.init.InitMilestone.*;
+
 /**
  * Manages {@link PluginWrapper}s.
  *
  * @author Kohsuke Kawaguchi
  */
+@ExportedBean
 public abstract class PluginManager extends AbstractModelObject {
     /**
      * All discovered plugins.
@@ -91,7 +104,7 @@ public abstract class PluginManager extends AbstractModelObject {
     protected final List<PluginWrapper> plugins = new ArrayList<PluginWrapper>();
 
     /**
-     * All active plugins.
+     * All active plugins, topologically sorted so that when X depends on Y, Y appears in the list before X does.
      */
     protected final List<PluginWrapper> activePlugins = new CopyOnWriteArrayList<PluginWrapper>();
 
@@ -120,7 +133,7 @@ public abstract class PluginManager extends AbstractModelObject {
 
     /**
      * Once plugin is uploaded, this flag becomes true.
-     * This is used to report a message that Hudson needs to be restarted
+     * This is used to report a message that Jenkins needs to be restarted
      * for new plugins to take effect.
      */
     public volatile boolean pluginUploaded = false;
@@ -148,6 +161,10 @@ public abstract class PluginManager extends AbstractModelObject {
         strategy = createPluginStrategy();
     }
 
+    public Api getApi() {
+        return new Api(this);
+    }
+
     /**
      * Called immediately after the construction.
      * This is a separate method so that code executed from here will see a valid value in
@@ -187,7 +204,7 @@ public abstract class PluginManager extends AbstractModelObject {
                                             PluginWrapper p = strategy.createPluginWrapper(arc);
                                             if (isDuplicate(p)) return;
 
-                                            p.isBundled = bundledPlugins.contains(arc.getName());
+                                            p.isBundled = containsHpiJpi(bundledPlugins, arc.getName());
                                             plugins.add(p);
                                         } catch (IOException e) {
                                             failedPlugins.add(new FailedPlugin(arc.getName(),e));
@@ -197,7 +214,7 @@ public abstract class PluginManager extends AbstractModelObject {
 
                                     /**
                                      * Inspects duplication. this happens when you run hpi:run on a bundled plugin,
-                                     * as well as putting numbered hpi files, like "cobertura-1.0.hpi" and "cobertura-1.1.hpi"
+                                     * as well as putting numbered jpi files, like "cobertura-1.0.jpi" and "cobertura-1.1.jpi"
                                      */
                                     private boolean isDuplicate(PluginWrapper p) {
                                         String shortName = p.getShortName();
@@ -234,6 +251,18 @@ public abstract class PluginManager extends AbstractModelObject {
                                                         r.add(p);
                                                 }
                                             }
+                                            
+                                            @Override
+                                            protected void reactOnCycle(PluginWrapper q, List<PluginWrapper> cycle)
+                                                    throws hudson.util.CyclicGraphDetector.CycleDetectedException {
+                                                
+                                                LOGGER.log(Level.SEVERE, "found cycle in plugin dependencies: (root="+q+", deactivating all involved) "+Util.join(cycle," -> "));
+                                                for (PluginWrapper pluginWrapper : cycle) {
+                                                    pluginWrapper.setHasCycleDependency(true);
+                                                    failedPlugins.add(new FailedPlugin(pluginWrapper.getShortName(), new CycleDetectedException(cycle)));
+                                                }
+                                            }
+                                            
                                         };
                                         cgd.run(getPlugins());
 
@@ -323,25 +352,89 @@ public abstract class PluginManager extends AbstractModelObject {
         }});
     }
 
+    /*
+     * contains operation that considers xxx.hpi and xxx.jpi as equal
+     * this is necessary since the bundled plugins are still called *.hpi 
+     */
+    private boolean containsHpiJpi(Collection<String> bundledPlugins, String name) {
+        return bundledPlugins.contains(name.replaceAll("\\.hpi",".jpi"))
+                || bundledPlugins.contains(name.replaceAll("\\.jpi",".hpi"));
+    }
+
     /**
-     * If the war file has any "/WEB-INF/plugins/*.hpi", extract them into the plugin directory.
+     * TODO: revisit where/how to expose this. This is an experiment.
+     */
+    public void dynamicLoad(File arc) throws IOException, InterruptedException, RestartRequiredException {
+        LOGGER.info("Attempting to dynamic load "+arc);
+        final PluginWrapper p = strategy.createPluginWrapper(arc);
+        String sn = p.getShortName();
+        if (getPlugin(sn)!=null)
+            throw new RestartRequiredException(Messages._PluginManager_PluginIsAlreadyInstalled_RestartRequired(sn));
+
+        if (p.supportsDynamicLoad()== YesNoMaybe.NO)
+            throw new RestartRequiredException(Messages._PluginManager_PluginDoesntSupportDynamicLoad_RestartRequired(sn));
+
+        // there's no need to do cyclic dependency check, because we are deploying one at a time,
+        // so existing plugins can't be depending on this newly deployed one.
+
+        plugins.add(p);
+        activePlugins.add(p);
+
+        try {
+            p.resolvePluginDependencies();
+            strategy.load(p);
+
+            Jenkins.getInstance().refreshExtensions();
+
+            p.getPlugin().postInitialize();
+        } catch (Exception e) {
+            failedPlugins.add(new FailedPlugin(sn, e));
+            activePlugins.remove(p);
+            plugins.remove(p);
+            throw new IOException2("Failed to install "+ sn +" plugin",e);
+        }
+
+        // run initializers in the added plugin
+        Reactor r = new Reactor(InitMilestone.ordering());
+        r.addAll(new InitializerFinder(p.classLoader) {
+            @Override
+            protected boolean filter(Method e) {
+                return e.getDeclaringClass().getClassLoader()!=p.classLoader || super.filter(e);
+            }
+        }.discoverTasks(r));
+        try {
+            new InitReactorRunner().run(r);
+        } catch (ReactorException e) {
+            throw new IOException2("Failed to initialize "+ sn +" plugin",e);
+        }
+        LOGGER.info("Plugin " + sn + " dynamically installed");
+    }
+
+    /**
+     * If the war file has any "/WEB-INF/plugins/[*.jpi | *.hpi]", extract them into the plugin directory.
      *
      * @return
-     *      File names of the bundled plugins. Like {"ssh-slaves.hpi","subvesrion.hpi"}
+     *      File names of the bundled plugins. Like {"ssh-slaves.hpi","subvesrion.jpi"}
      * @throws Exception
      *      Any exception will be reported and halt the startup.
      */
     protected abstract Collection<String> loadBundledPlugins() throws Exception;
 
     /**
-     * Copies the bundled plugin from the given URL to the destination of the given file name (like 'abc.hpi'),
+     * Copies the bundled plugin from the given URL to the destination of the given file name (like 'abc.jpi'),
      * with a reasonable up-to-date check. A convenience method to be used by the {@link #loadBundledPlugins()}.
      */
     protected void copyBundledPlugin(URL src, String fileName) throws IOException {
+        fileName = fileName.replace(".hpi",".jpi"); // normalize fileNames to have the correct suffix
+        String legacyName = fileName.replace(".jpi",".hpi");
         long lastModified = src.openConnection().getLastModified();
         File file = new File(rootDir, fileName);
         File pinFile = new File(rootDir, fileName+".pinned");
 
+        // normalization first, if the old file exists.
+        rename(new File(rootDir,legacyName),file);
+        rename(new File(rootDir,legacyName+".pinned"),pinFile);
+        
         // update file if:
         //  - no file exists today
         //  - bundled version and current version differs (by timestamp), and the file isn't pinned.
@@ -355,10 +448,24 @@ public abstract class PluginManager extends AbstractModelObject {
         }
     }
 
+    /**
+     * Rename a legacy file to a new name, with care to Windows where {@link File#renameTo(File)}
+     * doesn't work if the destination already exists.
+     */
+    private void rename(File legacyFile, File newFile) throws IOException {
+        if (!legacyFile.exists())   return;
+        if (newFile.exists()) {
+            Util.deleteFile(newFile);
+        }
+        if (!legacyFile.renameTo(newFile)) {
+            LOGGER.warning("Failed to rename " + legacyFile + " to " + newFile);
+        }
+    }
+
     /**
      * Creates a hudson.PluginStrategy, looking at the corresponding system property. 
      */
-    private PluginStrategy createPluginStrategy() {
+    protected PluginStrategy createPluginStrategy() {
 		String strategyName = System.getProperty(PluginStrategy.class.getName());
 		if (strategyName != null) {
 			try {
@@ -397,7 +504,11 @@ public abstract class PluginManager extends AbstractModelObject {
     public boolean isPluginUploaded() {
         return pluginUploaded;
     }
-    
+
+    /**
+     * All discovered plugins.
+     */
+    @Exported
     public List<PluginWrapper> getPlugins() {
         return plugins;
     }
@@ -471,12 +582,16 @@ public abstract class PluginManager extends AbstractModelObject {
      * @since 1.402.
      */
     public PluginWrapper whichPlugin(Class c) {
+        PluginWrapper oneAndOnly = null;
         ClassLoader cl = c.getClassLoader();
         for (PluginWrapper p : activePlugins) {
-            if (p.classLoader==cl)
-                return p;
+            if (p.classLoader==cl) {
+                if (oneAndOnly!=null)
+                    return null;    // ambigious
+                oneAndOnly = p;
+            }
         }
-        return null;
+        return oneAndOnly;
     }
 
     /**
@@ -516,6 +631,8 @@ public abstract class PluginManager extends AbstractModelObject {
      * Performs the installation of the plugins.
      */
     public void doInstall(StaplerRequest req, StaplerResponse rsp) throws IOException, ServletException {
+        boolean dynamicLoad = req.getParameter("dynamicLoad")!=null;
+
         Enumeration<String> en = req.getParameterNames();
         while (en.hasMoreElements()) {
             String n =  en.nextElement();
@@ -526,7 +643,7 @@ public abstract class PluginManager extends AbstractModelObject {
                     UpdateSite.Plugin p = Jenkins.getInstance().getUpdateCenter().getById(pluginInfo[1]).getPlugin(pluginInfo[0]);
                     if(p==null)
                         throw new Failure("No such plugin: "+n);
-                    p.deploy();
+                    p.deploy(dynamicLoad);
                 }
             }
         }
@@ -552,29 +669,21 @@ public abstract class PluginManager extends AbstractModelObject {
     }
 
 
-    public HttpResponse doProxyConfigure(
-            @QueryParameter("proxy.server") String server,
-            @QueryParameter("proxy.port") String port,
-            @QueryParameter("proxy.userName") String userName,
-            @QueryParameter("proxy.password") String password) throws IOException {
-        Jenkins hudson = Jenkins.getInstance();
-        hudson.checkPermission(Jenkins.ADMINISTER);
+    public HttpResponse doProxyConfigure(StaplerRequest req) throws IOException, ServletException {
+        Jenkins jenkins = Jenkins.getInstance();
+        jenkins.checkPermission(Jenkins.ADMINISTER);
 
-        server = Util.fixEmptyAndTrim(server);
-        if(server==null) {
-            hudson.proxy = null;
+        ProxyConfiguration pc = req.bindJSON(ProxyConfiguration.class, req.getSubmittedForm());
+        if (pc.name==null) {
+            jenkins.proxy = null;
             ProxyConfiguration.getXmlFile().delete();
-        } else try {
-            hudson.proxy = new ProxyConfiguration(server,Integer.parseInt(Util.fixNull(port)),
-                    Util.fixEmptyAndTrim(userName),Util.fixEmptyAndTrim(password));
-            hudson.proxy.save();
-        } catch (NumberFormatException nfe) {
-            return HttpResponses.error(StaplerResponse.SC_BAD_REQUEST,
-                    new IllegalArgumentException("Invalid proxy port: " + port, nfe));
+        } else {
+            jenkins.proxy = pc;
+            jenkins.proxy.save();
         }
         return new HttpRedirect("advanced");
     }
-
+    
     /**
      * Uploads a plugin.
      */
@@ -587,16 +696,21 @@ public abstract class PluginManager extends AbstractModelObject {
             // Parse the request
             FileItem fileItem = (FileItem) upload.parseRequest(req).get(0);
             String fileName = Util.getFileName(fileItem.getName());
-            if("".equals(fileName))
+            if("".equals(fileName)){
                 return new HttpRedirect("advanced");
-            if(!fileName.endsWith(".hpi"))
+            }
+            // we allow the upload of the new jpi's and the legacy hpi's  
+            if(!fileName.endsWith(".jpi") && !fileName.endsWith(".hpi")){ 
                 throw new Failure(hudson.model.Messages.Hudson_NotAPlugin(fileName));
-            fileItem.write(new File(rootDir, fileName));
+            }
+            final String baseName = FilenameUtils.getBaseName(fileName);
+            fileItem.write(new File(rootDir, baseName + ".jpi")); // rename all new plugins to *.jpi
             fileItem.delete();
 
-            PluginWrapper existing = getPlugin(FilenameUtils.getBaseName(fileName));
-            if (existing!=null && existing.isBundled)
+            PluginWrapper existing = getPlugin(baseName);
+            if (existing!=null && existing.isBundled){
                 existing.doPin();
+            }
 
             pluginUploaded = true;
 
@@ -608,6 +722,10 @@ public abstract class PluginManager extends AbstractModelObject {
         }
     }
 
+    public Descriptor<ProxyConfiguration> getProxyDescriptor() {
+        return Jenkins.getInstance().getDescriptor(ProxyConfiguration.class);
+    }
+
     /**
      * {@link ClassLoader} that can see all plugins.
      */
@@ -618,6 +736,8 @@ public abstract class PluginManager extends AbstractModelObject {
          */
         private ConcurrentMap<String, WeakReference<Class>> generatedClasses = new ConcurrentHashMap<String, WeakReference<Class>>();
 
+        private ClassLoaderReflectionToolkit clt = new ClassLoaderReflectionToolkit();
+
         public UberClassLoader() {
             super(PluginManager.class.getClassLoader());
         }
@@ -635,21 +755,24 @@ public abstract class PluginManager extends AbstractModelObject {
                 else            generatedClasses.remove(name,wc);
             }
 
-            // first, use the context classloader so that plugins that are loading
-            // can use its own classloader first.
-            ClassLoader cl = Thread.currentThread().getContextClassLoader();
-            if(cl!=null && cl!=this)
-                try {
-                    return cl.loadClass(name);
-                } catch(ClassNotFoundException e) {
-                    // not found. try next
+            if (FAST_LOOKUP) {
+                for (PluginWrapper p : activePlugins) {
+                    try {
+                        Class c = clt.findLoadedClass(p.classLoader,name);
+                        if (c!=null)    return c;
+                        // calling findClass twice appears to cause LinkageError: duplicate class def
+                        return clt.findClass(p.classLoader,name);
+                    } catch (InvocationTargetException e) {
+                        //not found. try next
+                    }
                 }
-
-            for (PluginWrapper p : activePlugins) {
-                try {
-                    return p.classLoader.loadClass(name);
-                } catch (ClassNotFoundException e) {
-                    //not found. try next
+            } else {
+                for (PluginWrapper p : activePlugins) {
+                    try {
+                        return p.classLoader.loadClass(name);
+                    } catch (ClassNotFoundException e) {
+                        //not found. try next
+                    }
                 }
             }
             // not found in any of the classloader. delegate.
@@ -658,10 +781,22 @@ public abstract class PluginManager extends AbstractModelObject {
 
         @Override
         protected URL findResource(String name) {
-            for (PluginWrapper p : activePlugins) {
-                URL url = p.classLoader.getResource(name);
-                if(url!=null)
-                    return url;
+            if (FAST_LOOKUP) {
+                try {
+                    for (PluginWrapper p : activePlugins) {
+                        URL url = clt.findResource(p.classLoader,name);
+                        if(url!=null)
+                            return url;
+                    }
+                } catch (InvocationTargetException e) {
+                    throw new Error(e);
+                }
+            } else {
+                for (PluginWrapper p : activePlugins) {
+                    URL url = p.classLoader.getResource(name);
+                    if(url!=null)
+                        return url;
+                }
             }
             return null;
         }
@@ -669,15 +804,24 @@ public abstract class PluginManager extends AbstractModelObject {
         @Override
         protected Enumeration<URL> findResources(String name) throws IOException {
             List<URL> resources = new ArrayList<URL>();
-            for (PluginWrapper p : activePlugins) {
-                resources.addAll(Collections.list(p.classLoader.getResources(name)));
+            if (FAST_LOOKUP) {
+                try {
+                    for (PluginWrapper p : activePlugins) {
+                        resources.addAll(Collections.list(clt.findResources(p.classLoader, name)));
+                    }
+                } catch (InvocationTargetException e) {
+                    throw new Error(e);
+                }
+            } else {
+                for (PluginWrapper p : activePlugins) {
+                    resources.addAll(Collections.list(p.classLoader.getResources(name)));
+                }
             }
             return Collections.enumeration(resources);
         }
 
         @Override
-        public String toString()
-        {
+        public String toString() {
             // only for debugging purpose
             return "classLoader " +  getClass().getName();
         }
@@ -685,6 +829,8 @@ public abstract class PluginManager extends AbstractModelObject {
 
     private static final Logger LOGGER = Logger.getLogger(PluginManager.class.getName());
 
+    public static boolean FAST_LOOKUP = !Boolean.getBoolean(PluginManager.class.getName()+".noFastLookup");
+
     /**
      * Remembers why a plugin failed to deploy.
      */
@@ -708,4 +854,32 @@ public abstract class PluginManager extends AbstractModelObject {
     /*package*/ static final class PluginInstanceStore {
         final Map<PluginWrapper,Plugin> store = new Hashtable<PluginWrapper,Plugin>();
     }
+    
+    /**
+     * {@link AdministrativeMonitor} that checks if there are any plugins with cycle dependencies.
+     */
+    @Extension
+    public static final class PluginCycleDependenciesMonitor extends AdministrativeMonitor {
+        
+        private transient volatile boolean isActive = false;
+        
+        private transient volatile List<String> pluginsWithCycle; 
+        
+        public boolean isActivated() {
+            if(pluginsWithCycle == null){
+                pluginsWithCycle = new ArrayList<String>();
+                for (PluginWrapper p : Jenkins.getInstance().getPluginManager().getPlugins()) {
+                    if(p.hasCycleDependency()){
+                        pluginsWithCycle.add(p.getShortName());
+                        isActive = true;
+                    }
+                }
+            }
+            return isActive;
+        }
+
+        public List<String> getPluginsWithCycle() {
+            return pluginsWithCycle;
+        }
+    }
 }
