@@ -25,6 +25,8 @@
 package hudson;
 
 import hudson.PluginManager.PluginInstanceStore;
+import hudson.model.Api;
+import jenkins.YesNoMaybe;
 import jenkins.model.Jenkins;
 import hudson.model.UpdateCenter;
 import hudson.model.UpdateSite;
@@ -45,6 +47,8 @@ import static java.util.logging.Level.WARNING;
 import org.apache.commons.logging.LogFactory;
 import org.kohsuke.stapler.HttpResponse;
 import org.kohsuke.stapler.HttpResponses;
+import org.kohsuke.stapler.export.Exported;
+import org.kohsuke.stapler.export.ExportedBean;
 
 import java.util.Enumeration;
 import java.util.jar.JarFile;
@@ -54,7 +58,7 @@ import java.util.jar.JarFile;
  * for Jenkins to control {@link Plugin}.
  *
  * <p>
- * A plug-in is packaged into a jar file whose extension is <tt>".hpi"</tt>,
+ * A plug-in is packaged into a jar file whose extension is <tt>".jpi"</tt> (or <tt>".hpi"</tt> for backward compatability),
  * A plugin needs to have a special manifest entry to identify what it is.
  *
  * <p>
@@ -71,6 +75,7 @@ import java.util.jar.JarFile;
  *
  * @author Kohsuke Kawaguchi
  */
+@ExportedBean
 public class PluginWrapper implements Comparable<PluginWrapper> {
     /**
      * {@link PluginManager} to which this belongs to.
@@ -92,7 +97,7 @@ public class PluginWrapper implements Comparable<PluginWrapper> {
     /**
      * Base URL for loading static resources from this plugin.
      * Null if disabled. The static resources are mapped under
-     * <tt>hudson/plugin/SHORTNAME/</tt>.
+     * <tt>CONTEXTPATH/plugin/SHORTNAME/</tt>.
      */
     public final URL baseResourceURL;
 
@@ -114,7 +119,7 @@ public class PluginWrapper implements Comparable<PluginWrapper> {
     /**
      * Short name of the plugin. The artifact Id of the plugin.
      * This is also used in the URL within Jenkins, so it needs
-     * to remain stable even when the *.hpi file name is changed
+     * to remain stable even when the *.jpi file name is changed
      * (like Maven does.)
      */
     private final String shortName;
@@ -124,6 +129,8 @@ public class PluginWrapper implements Comparable<PluginWrapper> {
      * The snapshot of <tt>disableFile.exists()</tt> as of the start up.
      */
     private final boolean active;
+    
+    private boolean hasCycleDependency = false;
 
     private final List<Dependency> dependencies;
     private final List<Dependency> optionalDependencies;
@@ -133,9 +140,13 @@ public class PluginWrapper implements Comparable<PluginWrapper> {
      */
     /*package*/ boolean isBundled;
 
-    static final class Dependency {
+    @ExportedBean
+    public static final class Dependency {
+        @Exported
         public final String shortName;
+        @Exported
         public final String version;
+        @Exported
         public final boolean optional;
 
         public Dependency(String s) {
@@ -164,7 +175,7 @@ public class PluginWrapper implements Comparable<PluginWrapper> {
 
     /**
      * @param archive
-     *      A .hpi archive file jar file, or a .hpl linked plugin.
+     *      A .jpi archive file jar file, or a .jpl linked plugin.
      *  @param manifest
      *  	The manifest for the plugin
      *  @param baseResourceURL
@@ -191,6 +202,10 @@ public class PluginWrapper implements Comparable<PluginWrapper> {
 		this.optionalDependencies = optionalDependencies;
     }
 
+    public Api getApi() {
+        return new Api(this);
+    }
+
     /**
      * Returns the URL of the index page jelly script.
      */
@@ -235,6 +250,7 @@ public class PluginWrapper implements Comparable<PluginWrapper> {
         return n;
     }
 
+    @Exported
     public List<Dependency> getDependencies() {
         return dependencies;
     }
@@ -247,6 +263,7 @@ public class PluginWrapper implements Comparable<PluginWrapper> {
     /**
      * Returns the short name suitable for URL.
      */
+    @Exported
     public String getShortName() {
         return shortName;
     }
@@ -264,6 +281,7 @@ public class PluginWrapper implements Comparable<PluginWrapper> {
      *      null if this information is unavailable.
      * @since 1.283
      */
+    @Exported
     public String getUrl() {
         // first look for the manifest entry. This is new in maven-hpi-plugin 1.30
         String url = manifest.getMainAttributes().getValue("Url");
@@ -275,6 +293,8 @@ public class PluginWrapper implements Comparable<PluginWrapper> {
 
         return null;
     }
+    
+    
 
     @Override
     public String toString() {
@@ -284,15 +304,27 @@ public class PluginWrapper implements Comparable<PluginWrapper> {
     /**
      * Returns a one-line descriptive name of this plugin.
      */
+    @Exported
     public String getLongName() {
         String name = manifest.getMainAttributes().getValue("Long-Name");
         if(name!=null)      return name;
         return shortName;
     }
 
+    /**
+     * Does this plugin supports dynamic loading?
+     */
+    @Exported
+    public YesNoMaybe supportsDynamicLoad() {
+        String v = manifest.getMainAttributes().getValue("Support-Dynamic-Loading");
+        if (v==null) return YesNoMaybe.MAYBE;
+        return Boolean.parseBoolean(v) ? YesNoMaybe.YES : YesNoMaybe.NO;
+    }
+
     /**
      * Returns the version number of this plugin
      */
+    @Exported
     public String getVersion() {
         String v = manifest.getMainAttributes().getValue("Plugin-Version");
         if(v!=null)      return v;
@@ -368,10 +400,20 @@ public class PluginWrapper implements Comparable<PluginWrapper> {
     /**
      * Returns true if this plugin is enabled for this session.
      */
+    @Exported
     public boolean isActive() {
-        return active;
+        return active && !hasCycleDependency();
+    }
+    
+    public boolean hasCycleDependency(){
+        return hasCycleDependency;
     }
 
+    public void setHasCycleDependency(boolean hasCycle){
+        hasCycleDependency = hasCycle;
+    }
+    
+    @Exported
     public boolean isBundled() {
         return isBundled;
     }
@@ -380,6 +422,7 @@ public class PluginWrapper implements Comparable<PluginWrapper> {
      * If true, the plugin is going to be activated next time
      * Jenkins runs.
      */
+    @Exported
     public boolean isEnabled() {
         return !disableFile.exists();
     }
@@ -460,10 +503,12 @@ public class PluginWrapper implements Comparable<PluginWrapper> {
      * This method is conservative in the sense that if the version number is incomprehensible,
      * it always returns false.
      */
+    @Exported
     public boolean hasUpdate() {
         return getUpdateInfo()!=null;
     }
     
+    @Exported
     public boolean isPinned() {
         return pinFile.exists();
     }
@@ -478,6 +523,7 @@ public class PluginWrapper implements Comparable<PluginWrapper> {
     /**
      * returns true if backup of previous version of plugin exists
      */
+    @Exported
     public boolean isDowngradable() {
         return getBackupFile().exists();
     }
@@ -493,6 +539,7 @@ public class PluginWrapper implements Comparable<PluginWrapper> {
      * returns the version of the backed up plugin,
      * or null if there's no back up.
      */
+    @Exported
     public String getBackupVersion() {
         File backup = getBackupFile();
         if (backup.exists()) {
