@@ -217,14 +217,8 @@ public final class WebAppMain implements ServletContextListener {
                         Jenkins instance = new Hudson(home, context);
                         context.setAttribute(APP, instance);
 
-                        // trigger the loading of changelogs in the background,
-                        // but give the system 10 seconds so that the first page
-                        // can be served quickly
-                        Trigger.timer.schedule(new SafeTimerTask() {
-                            public void doRun() {
-                                User.getUnknown().getBuilds();
-                            }
-                        }, 1000*10);
+                        // at this point we are open for business and serving requests normally
+                        LOGGER.info("Jenkins is fully up and running");
                         success = true;
                     } catch (Error e) {
                         LOGGER.log(Level.SEVERE, "Failed to initialize Jenkins",e);
@@ -257,6 +251,7 @@ public final class WebAppMain implements ServletContextListener {
 	/**
      * Installs log handler to monitor all Hudson logs.
      */
+    @edu.umd.cs.findbugs.annotations.SuppressWarnings("LG_LOST_LOGGER_DUE_TO_WEAK_REFERENCE")
     private void installLogger() {
         Jenkins.logRecords = handler.getView();
         Logger.getLogger("hudson").addHandler(handler);
@@ -264,9 +259,9 @@ public final class WebAppMain implements ServletContextListener {
     }
 
     /** Add some metadata to a File, allowing to trace setup issues */
-    private static class FileAndDescription {
-        File file;
-        String description;
+    public static class FileAndDescription {
+        public final File file;
+        public final String description;
         public FileAndDescription(File file,String description) {
             this.file = file;
             this.description = description;
@@ -286,7 +281,7 @@ public final class WebAppMain implements ServletContextListener {
      * <p>
      * @return the File alongside with some description to help the user troubleshoot issues
      */
-    private FileAndDescription getHomeDir(ServletContextEvent event) {
+    public FileAndDescription getHomeDir(ServletContextEvent event) {
         // check JNDI for the home directory first
         for (String name : HOME_NAMES) {
             try {
