@@ -24,20 +24,21 @@
  */
 package hudson.matrix;
 
+import hudson.AbortException;
 import hudson.Util;
-import hudson.matrix.listeners.MatrixBuildListener;
+import hudson.console.HyperlinkNote;
 import hudson.model.AbstractBuild;
 import hudson.model.AbstractProject;
 import hudson.model.BuildListener;
 import hudson.model.Executor;
 import hudson.model.Fingerprint;
-import jenkins.model.Jenkins;
-import hudson.model.JobProperty;
-import hudson.model.ParametersAction;
 import hudson.model.Queue;
 import hudson.model.Result;
-import hudson.model.Cause.UpstreamCause;
-import hudson.tasks.Publisher;
+import jenkins.model.Jenkins;
+import org.kohsuke.stapler.StaplerRequest;
+import org.kohsuke.stapler.StaplerResponse;
+import org.kohsuke.stapler.export.Exported;
+import org.kohsuke.stapler.interceptor.RequirePOST;
 
 import java.io.File;
 import java.io.IOException;
@@ -45,12 +46,9 @@ import java.io.PrintStream;
 import java.util.ArrayList;
 import java.util.Calendar;
 import java.util.Collection;
-import java.util.HashSet;
 import java.util.List;
 
-import org.kohsuke.stapler.StaplerRequest;
-import org.kohsuke.stapler.StaplerResponse;
-import org.kohsuke.stapler.export.Exported;
+import javax.servlet.ServletException;
 
 /**
  * Build of {@link MatrixProject}.
@@ -85,6 +83,36 @@ public class MatrixBuild extends AbstractBuild<MatrixProject,MatrixBuild> {
         return this;
     }
 
+    /**
+     * Deletes the build and all matrix configurations in this build when the button is pressed.
+     */
+    @RequirePOST
+    public void doDoDeleteAll( StaplerRequest req, StaplerResponse rsp ) throws IOException, ServletException {
+        checkPermission(DELETE);
+
+        // We should not simply delete the build if it has been explicitly
+        // marked to be preserved, or if the build should not be deleted
+        // due to dependencies!
+        String why = getWhyKeepLog();
+        if (why!=null) {
+            sendError(hudson.model.Messages.Run_UnableToDelete(toString(),why),req,rsp);
+            return;
+        }
+        
+        List<MatrixRun> runs = getRuns();
+        for(MatrixRun run : runs){
+        	why = run.getWhyKeepLog();
+            if (why!=null) {
+                sendError(hudson.model.Messages.Run_UnableToDelete(toString(),why),req,rsp);
+                return;
+            }
+        	run.delete();
+        }
+        delete();
+        rsp.sendRedirect2(req.getContextPath()+'/' + getParent().getUrl());
+    }
+
+    
     /**
      * Used by view to render a ball for {@link MatrixRun}.
      */
@@ -195,7 +223,7 @@ public class MatrixBuild extends AbstractBuild<MatrixProject,MatrixBuild> {
      * True if this build didn't do a full build and it is depending on the result of the previous build.
      */
     public boolean isPartial() {
-        for(MatrixConfiguration c : getParent().getItems()) {
+        for(MatrixConfiguration c : getParent().getActiveConfigurations()) {
             MatrixRun b = c.getNearestOldBuild(getNumber());
             if (b != null && b.getNumber()!=getNumber())
                 return true;
@@ -236,94 +264,35 @@ public class MatrixBuild extends AbstractBuild<MatrixProject,MatrixBuild> {
             PrintStream logger = listener.getLogger();
 
             // list up aggregators
-            for (Publisher pub : p.getPublishers().values()) {
-                if (pub instanceof MatrixAggregatable) {
-                    MatrixAggregatable ma = (MatrixAggregatable) pub;
-                    MatrixAggregator a = ma.createAggregator(MatrixBuild.this, launcher, listener);
-                    if(a!=null)
-                        aggregators.add(a);
-                }
-            }
-
-            //let properties do their job
-            for (JobProperty prop : p.getProperties().values()) {
-                if (prop instanceof MatrixAggregatable) {
-                    MatrixAggregatable ma = (MatrixAggregatable) prop;
-                    MatrixAggregator a = ma.createAggregator(MatrixBuild.this, launcher, listener);
-                    if(a!=null)
-                        aggregators.add(a);
-                }
-            }
+            listUpAggregators(listener, p.getPublishers().values());
+            listUpAggregators(listener, p.getProperties().values());
+            listUpAggregators(listener, p.getBuildWrappers().values());
 
             axes = p.getAxes();
-            Collection<MatrixConfiguration> activeConfigurations = p.getActiveConfigurations();
-            final int n = getNumber();
-            
-            String touchStoneFilter = p.getTouchStoneCombinationFilter();
-            Collection<MatrixConfiguration> touchStoneConfigurations = new HashSet<MatrixConfiguration>();
-            Collection<MatrixConfiguration> delayedConfigurations = new HashSet<MatrixConfiguration>();
-            for (MatrixConfiguration c: activeConfigurations) {
-                if (!MatrixBuildListener.buildConfiguration(MatrixBuild.this, c))
-                    continue; // skip rebuild
-                if (touchStoneFilter != null && c.getCombination().evalGroovyExpression(p.getAxes(), p.getTouchStoneCombinationFilter())) {
-                    touchStoneConfigurations.add(c);
-                } else {
-                    delayedConfigurations.add(c);
-                }
-            }
-
-            for (MatrixAggregator a : aggregators)
-                if(!a.startBuild())
-                    return Result.FAILURE;
 
             try {
-                if(!p.isRunSequentially())
-                    for(MatrixConfiguration c : touchStoneConfigurations)
-                        scheduleConfigurationBuild(logger, c);
-
-                Result r = Result.SUCCESS;
-                for (MatrixConfiguration c : touchStoneConfigurations) {
-                    if(p.isRunSequentially())
-                        scheduleConfigurationBuild(logger, c);
-                    Result buildResult = waitForCompletion(listener, c);
-                    r = r.combine(buildResult);
-                }
-                
-                if (p.getTouchStoneResultCondition() != null && r.isWorseThan(p.getTouchStoneResultCondition())) {
-                    logger.printf("Touchstone configurations resulted in %s, so aborting...%n", r);
-                    return r;
-                }
-                
-                if(!p.isRunSequentially())
-                    for(MatrixConfiguration c : delayedConfigurations)
-                        scheduleConfigurationBuild(logger, c);
-
-                for (MatrixConfiguration c : delayedConfigurations) {
-                    if(p.isRunSequentially())
-                        scheduleConfigurationBuild(logger, c);
-                    Result buildResult = waitForCompletion(listener, c);
-                    r = r.combine(buildResult);
-                }
-
-                return r;
+                return p.getExecutionStrategy().run(MatrixBuild.this, aggregators, listener);
             } catch( InterruptedException e ) {
                 logger.println("Aborted");
-                return Executor.currentExecutor().abortResult();
-            } catch (AggregatorFailureException e) {
+                Executor x = Executor.currentExecutor();
+                x.recordCauseOfInterruption(MatrixBuild.this, listener);
+                return x.abortResult();
+            } catch (AbortException e) {
+                logger.println(e.getMessage());
                 return Result.FAILURE;
-            }
-            finally {
+            } finally {
                 // if the build was aborted in the middle. Cancel all the configuration builds.
                 Queue q = Jenkins.getInstance().getQueue();
                 synchronized(q) {// avoid micro-locking in q.cancel.
-                    for (MatrixConfiguration c : activeConfigurations) {
+                    final int n = getNumber();
+                    for (MatrixConfiguration c : p.getActiveConfigurations()) {
                         if(q.cancel(c))
-                            logger.println(Messages.MatrixBuild_Cancelled(c.getDisplayName()));
+                            logger.println(Messages.MatrixBuild_Cancelled(HyperlinkNote.encodeTo('/'+ c.getUrl(),c.getDisplayName())));
                         MatrixRun b = c.getBuildByNumber(n);
-                        if(b!=null) {
+                        if(b!=null && b.isBuilding()) {// executor can spend some time in post production state, so only cancel in-progress builds.
                             Executor exe = b.getExecutor();
                             if(exe!=null) {
-                                logger.println(Messages.MatrixBuild_Interrupting(b.getDisplayName()));
+                                logger.println(Messages.MatrixBuild_Interrupting(HyperlinkNote.encodeTo('/'+ b.getUrl(),b.getDisplayName())));
                                 exe.interrupt();
                             }
                         }
@@ -331,73 +300,21 @@ public class MatrixBuild extends AbstractBuild<MatrixProject,MatrixBuild> {
                 }
             }
         }
-        
-        private Result waitForCompletion(BuildListener listener, MatrixConfiguration c) throws InterruptedException, IOException, AggregatorFailureException {
-            String whyInQueue = "";
-            long startTime = System.currentTimeMillis();
 
-            // wait for the completion
-            int appearsCancelledCount = 0;
-            while(true) {
-                MatrixRun b = c.getBuildByNumber(getNumber());
-
-                // two ways to get beyond this. one is that the build starts and gets done,
-                // or the build gets cancelled before it even started.
-                Result buildResult = null;
-                if(b!=null && !b.isBuilding())
-                    buildResult = b.getResult();
-                Queue.Item qi = c.getQueueItem();
-                if(b==null && qi==null)
-                    appearsCancelledCount++;
-                else
-                    appearsCancelledCount = 0;
-
-                if(appearsCancelledCount>=5) {
-                    // there's conceivably a race condition in computating b and qi, as their computation
-                    // are not synchronized. There are indeed several reports of Hudson incorrectly assuming
-                    // builds being cancelled. See
-                    // http://www.nabble.com/Master-slave-problem-tt14710987.html and also
-                    // http://www.nabble.com/Anyone-using-AccuRev-plugin--tt21634577.html#a21671389
-                    // because of this, we really make sure that the build is cancelled by doing this 5
-                    // times over 5 seconds
-                    listener.getLogger().println(Messages.MatrixBuild_AppearsCancelled(c.getDisplayName()));
-                    buildResult = Result.ABORTED;
+        private void listUpAggregators(BuildListener listener, Collection<?> values) {
+            for (Object v : values) {
+                if (v instanceof MatrixAggregatable) {
+                    MatrixAggregatable ma = (MatrixAggregatable) v;
+                    MatrixAggregator a = ma.createAggregator(MatrixBuild.this, launcher, listener);
+                    if(a!=null)
+                        aggregators.add(a);
                 }
-
-                if(buildResult!=null) {
-                    for (MatrixAggregator a : aggregators)
-                        if(!a.endRun(b))
-                            throw new AggregatorFailureException();
-                    return buildResult;
-                } 
-
-                if(qi!=null) {
-                    // if the build seems to be stuck in the queue, display why
-                    String why = qi.getWhy();
-                    if(!why.equals(whyInQueue) && System.currentTimeMillis()-startTime>5000) {
-                        listener.getLogger().println(c.getDisplayName()+" is still in the queue: "+why);
-                        whyInQueue = why;
-                    }
-                }
-                
-                Thread.sleep(1000);
             }
         }
 
-        private void scheduleConfigurationBuild(PrintStream logger, MatrixConfiguration c) {
-            logger.println(Messages.MatrixBuild_Triggering(c.getDisplayName()));
-            c.scheduleBuild(getAction(ParametersAction.class), new UpstreamCause(MatrixBuild.this));
-        }
-
         public void post2(BuildListener listener) throws Exception {
             for (MatrixAggregator a : aggregators)
                 a.endBuild();
         }
     }
-
-    /**
-     * A private exception to help maintain the correct control flow after extracting the 'waitForCompletion' method
-     */
-    private static class AggregatorFailureException extends Exception {}
-
 }
