@@ -42,7 +42,6 @@ import hudson.model.JDK;
 import hudson.model.Job;
 import hudson.model.Label;
 import hudson.model.Queue.FlyweightTask;
-import hudson.model.ResourceController;
 import hudson.model.Result;
 import hudson.model.SCMedItem;
 import hudson.model.Saveable;
@@ -131,18 +130,34 @@ public class MatrixProject extends AbstractProject<MatrixProject,MatrixBuild> im
     @CopyOnWrite
     private transient /*final*/ Set<MatrixConfiguration> activeConfigurations = new LinkedHashSet<MatrixConfiguration>();
 
-    private boolean runSequentially;
+    /**
+     * @deprecated as of 1.456
+     *      Moved to {@link DefaultMatrixExecutionStrategyImpl}
+     */
+    private transient Boolean runSequentially;
     
     /**
      * Filter to select a number of combinations to build first
+     * @deprecated as of 1.456
+     *      Moved to {@link DefaultMatrixExecutionStrategyImpl}
      */
-    private String touchStoneCombinationFilter;
+    private transient String touchStoneCombinationFilter;
     
     /**
      * Required result on the touchstone combinations, in order to
      * continue with the rest
+     * @deprecated as of 1.456
+     *      Moved to {@link DefaultMatrixExecutionStrategyImpl}
      */
-    private Result touchStoneResultCondition;
+    private transient Result touchStoneResultCondition;
+
+    /**
+     * @deprecated as of 1.456
+     *      Moved to {@link DefaultMatrixExecutionStrategyImpl}
+     */
+    private transient MatrixConfigurationSorter sorter;
+
+    private MatrixExecutionStrategy executionStrategy;
 
     public MatrixProject(String name) {
         this(Jenkins.getInstance(), name);
@@ -152,6 +167,50 @@ public class MatrixProject extends AbstractProject<MatrixProject,MatrixBuild> im
         super(parent, name);
     }
 
+
+
+    /**
+     * {@link MatrixProject} is relevant with all the labels its configurations are relevant.
+     */
+    @Override
+    public Set<Label> getRelevantLabels() {
+        Set<Label> r = new HashSet<Label>();
+        for (MatrixConfiguration c : getActiveConfigurations())
+            r.add(c.getAssignedLabel());
+        return r;
+    }
+
+    /**
+     * @return can be null (to indicate that the configurations should be left to their natural order.)
+     * @deprecated as of 1.456
+     *      Use {@link DefaultMatrixExecutionStrategyImpl#getSorter()}.
+     *      This method tries to emulate the previous behavior the best it can, but will return null
+     *      if the current {@link MatrixExecutionStrategy} is not the default one.
+     */
+    public MatrixConfigurationSorter getSorter() {
+        MatrixExecutionStrategy e = executionStrategy;
+        if (e instanceof DefaultMatrixExecutionStrategyImpl) {
+            DefaultMatrixExecutionStrategyImpl dm = (DefaultMatrixExecutionStrategyImpl) e;
+            return dm.getSorter();
+        }
+        return null;
+    }
+
+    /**
+     * @deprecated as of 1.456
+     *      Use {@link DefaultMatrixExecutionStrategyImpl#setSorter(MatrixConfigurationSorter)}.
+     *      This method tries to emulate the previous behavior the best it can, but will fall back
+     *      to no-op if the current {@link MatrixExecutionStrategy} is not the default one.
+     */
+    public void setSorter(MatrixConfigurationSorter sorter) throws IOException {
+        MatrixExecutionStrategy e = executionStrategy;
+        if (e instanceof DefaultMatrixExecutionStrategyImpl) {
+            DefaultMatrixExecutionStrategyImpl dm = (DefaultMatrixExecutionStrategyImpl) e;
+            dm.setSorter(sorter);
+            save();
+        }
+    }
+    
     public AxisList getAxes() {
         return axes;
     }
@@ -165,20 +224,46 @@ public class MatrixProject extends AbstractProject<MatrixProject,MatrixBuild> im
         save();
     }
 
-    /**
-     * If true, {@link MatrixRun}s are run sequentially, instead of running in parallel.
-     *
-     * TODO: this should be subsumed by {@link ResourceController}.
-     */
-    public boolean isRunSequentially() {
-        return runSequentially;
+    public MatrixExecutionStrategy getExecutionStrategy() {
+        return executionStrategy;
     }
 
-    public void setRunSequentially(boolean runSequentially) throws IOException {
-        this.runSequentially = runSequentially;
+    public void setExecutionStrategy(MatrixExecutionStrategy executionStrategy) throws IOException {
+        if (executionStrategy ==null)   throw new IllegalArgumentException();
+        this.executionStrategy = executionStrategy;
         save();
     }
 
+    /**
+     * @deprecated as of 1.456
+     *      Use {@link DefaultMatrixExecutionStrategyImpl#isRunSequentially()}.
+     *      This method tries to emulate the previous behavior the best it can, but will return false
+     *      if the current {@link MatrixExecutionStrategy} is not the default one.
+     */
+    public boolean isRunSequentially() {
+        MatrixExecutionStrategy e = executionStrategy;
+        if (e instanceof DefaultMatrixExecutionStrategyImpl) {
+            DefaultMatrixExecutionStrategyImpl dm = (DefaultMatrixExecutionStrategyImpl) e;
+            return dm.isRunSequentially();
+        }
+        return false;
+    }
+
+    /**
+     * @deprecated as of 1.456
+     *      Use {@link DefaultMatrixExecutionStrategyImpl#setRunSequentially(boolean)}.
+     *      This method tries to emulate the previous behavior the best it can, but will fall back
+     *      to no-op if the current {@link MatrixExecutionStrategy} is not the default one.
+     */
+    public void setRunSequentially(boolean runSequentially) throws IOException {
+        MatrixExecutionStrategy e = executionStrategy;
+        if (e instanceof DefaultMatrixExecutionStrategyImpl) {
+            DefaultMatrixExecutionStrategyImpl dm = (DefaultMatrixExecutionStrategyImpl) e;
+            dm.setRunSequentially(runSequentially);
+            save();
+        }
+    }
+
     /**
      * Sets the combination filter.
      *
@@ -208,21 +293,66 @@ public class MatrixProject extends AbstractProject<MatrixProject,MatrixBuild> im
         return combinationFilter;
     }
 
+    /**
+     * @return can be null (to indicate that the configurations should be left to their natural order.)
+     * @deprecated as of 1.456
+     *      Use {@link DefaultMatrixExecutionStrategyImpl#getTouchStoneCombinationFilter()}.
+     *      This method tries to emulate the previous behavior the best it can, but will return null
+     *      if the current {@link MatrixExecutionStrategy} is not the default one.
+     */
     public String getTouchStoneCombinationFilter() {
-        return touchStoneCombinationFilter;
+        MatrixExecutionStrategy e = executionStrategy;
+        if (e instanceof DefaultMatrixExecutionStrategyImpl) {
+            DefaultMatrixExecutionStrategyImpl dm = (DefaultMatrixExecutionStrategyImpl) e;
+            return dm.getTouchStoneCombinationFilter();
+        }
+        return null;
     }
 
-    public void setTouchStoneCombinationFilter(
-            String touchStoneCombinationFilter) {
-        this.touchStoneCombinationFilter = touchStoneCombinationFilter;
+    /**
+     * @deprecated as of 1.456
+     *      Use {@link DefaultMatrixExecutionStrategyImpl#setTouchStoneCombinationFilter(String)}.
+     *      This method tries to emulate the previous behavior the best it can, but will fall back
+     *      to no-op if the current {@link MatrixExecutionStrategy} is not the default one.
+     */
+    public void setTouchStoneCombinationFilter(String touchStoneCombinationFilter) throws IOException {
+        MatrixExecutionStrategy e = executionStrategy;
+        if (e instanceof DefaultMatrixExecutionStrategyImpl) {
+            DefaultMatrixExecutionStrategyImpl dm = (DefaultMatrixExecutionStrategyImpl) e;
+            dm.setTouchStoneCombinationFilter(touchStoneCombinationFilter);
+            save();
+        }
     }
 
+    /**
+     * @return can be null (to indicate that the configurations should be left to their natural order.)
+     * @deprecated as of 1.456
+     *      Use {@link DefaultMatrixExecutionStrategyImpl#getTouchStoneResultCondition()}.
+     *      This method tries to emulate the previous behavior the best it can, but will return null
+     *      if the current {@link MatrixExecutionStrategy} is not the default one.
+     */
     public Result getTouchStoneResultCondition() {
-        return touchStoneResultCondition;
+        MatrixExecutionStrategy e = executionStrategy;
+        if (e instanceof DefaultMatrixExecutionStrategyImpl) {
+            DefaultMatrixExecutionStrategyImpl dm = (DefaultMatrixExecutionStrategyImpl) e;
+            return dm.getTouchStoneResultCondition();
+        }
+        return null;
     }
 
-    public void setTouchStoneResultCondition(Result touchStoneResultCondition) {
-        this.touchStoneResultCondition = touchStoneResultCondition;
+    /**
+     * @deprecated as of 1.456
+     *      Use {@link DefaultMatrixExecutionStrategyImpl#setTouchStoneResultCondition(Result)}.
+     *      This method tries to emulate the previous behavior the best it can, but will fall back
+     *      to no-op if the current {@link MatrixExecutionStrategy} is not the default one.
+     */
+    public void setTouchStoneResultCondition(Result touchStoneResultCondition) throws IOException {
+        MatrixExecutionStrategy e = executionStrategy;
+        if (e instanceof DefaultMatrixExecutionStrategyImpl) {
+            DefaultMatrixExecutionStrategyImpl dm = (DefaultMatrixExecutionStrategyImpl) e;
+            dm.setTouchStoneResultCondition(touchStoneResultCondition);
+            save();
+        }
     }
 
     @Override
@@ -235,7 +365,7 @@ public class MatrixProject extends AbstractProject<MatrixProject,MatrixBuild> im
             r.addAll(step.getProjectActions(this));
         for (BuildWrapper step : buildWrappers)
             r.addAll(step.getProjectActions(this));
-        for (Trigger trigger : triggers)
+        for (Trigger<?> trigger : triggers)
             r.addAll(trigger.getProjectActions());
 
         return r;
@@ -263,6 +393,12 @@ public class MatrixProject extends AbstractProject<MatrixProject,MatrixBuild> im
         };
     }
 
+    @Override
+    public void onCreatedFromScratch() {
+        executionStrategy = new DefaultMatrixExecutionStrategyImpl();
+        super.onCreatedFromScratch();
+    }
+
     @Override
     public void onLoad(ItemGroup<? extends Item> parent, String name) throws IOException {
         super.onLoad(parent,name);
@@ -270,6 +406,10 @@ public class MatrixProject extends AbstractProject<MatrixProject,MatrixBuild> im
         builders.setOwner(this);
         publishers.setOwner(this);
         buildWrappers.setOwner(this);
+
+        if (executionStrategy ==null)
+            executionStrategy = new DefaultMatrixExecutionStrategyImpl(runSequentially,touchStoneCombinationFilter,touchStoneResultCondition,sorter);
+
         rebuildConfigurations();
     }
 
@@ -425,7 +565,11 @@ public class MatrixProject extends AbstractProject<MatrixProject,MatrixBuild> im
     }
 
     public MatrixConfiguration getItem(String name) {
-        return getItem(Combination.fromString(name));
+        try {
+            return getItem(Combination.fromString(name));
+        } catch (IllegalArgumentException e) {
+            return null;
+        }
     }
 
     public MatrixConfiguration getItem(Combination c) {
@@ -560,26 +704,22 @@ public class MatrixProject extends AbstractProject<MatrixProject,MatrixBuild> im
         } else {
             this.combinationFilter = null;
         }
-        
-        if (req.getParameter("hasTouchStoneCombinationFilter")!=null) {
-            this.touchStoneCombinationFilter = Util.nullify(req.getParameter("touchStoneCombinationFilter"));
-            String touchStoneResultCondition = req.getParameter("touchStoneResultCondition");
-            this.touchStoneResultCondition = Result.fromString(touchStoneResultCondition);
-        } else {
-            this.touchStoneCombinationFilter = null;
-        }
+
+        List<MatrixExecutionStrategyDescriptor> esd = getDescriptor().getExecutionStrategyDescriptors();
+        if (esd.size()>1)
+            executionStrategy = req.bindJSON(MatrixExecutionStrategy.class,json.getJSONObject("executionStrategy"));
+        else
+            executionStrategy = req.bindJSON(esd.get(0).clazz,json.getJSONObject("executionStrategy"));
 
         // parse system axes
         DescribableList<Axis,AxisDescriptor> newAxes = new DescribableList<Axis,AxisDescriptor>(this);
         newAxes.rebuildHetero(req, json, Axis.all(),"axis");
         checkAxisNames(newAxes);
         this.axes = new AxisList(newAxes.toList());
-
-        runSequentially = json.has("runSequentially");
-
+        
         buildWrappers.rebuild(req, json, BuildWrappers.getFor(this));
         builders.rebuildHetero(req, json, Builder.all(), "builder");
-        publishers.rebuild(req, json, BuildStepDescriptor.filter(Publisher.all(),this.getClass()));
+        publishers.rebuildHetero(req, json, Publisher.all(), "publisher");
 
         rebuildConfigurations();
     }
@@ -639,6 +779,18 @@ public class MatrixProject extends AbstractProject<MatrixProject,MatrixBuild> im
             }
             return r;
         }
+
+        /**
+         * @deprecated as of 1.456
+         *      This was only exposed for Jelly.
+         */
+        public List<MatrixConfigurationSorterDescriptor> getSorterDescriptors() {
+            return MatrixConfigurationSorterDescriptor.all();
+        }
+
+        public List<MatrixExecutionStrategyDescriptor> getExecutionStrategyDescriptors() {
+            return MatrixExecutionStrategyDescriptor.all();
+        }
     }
 
     private static final Logger LOGGER = Logger.getLogger(MatrixProject.class.getName());
