@@ -27,8 +27,8 @@
  */
 package hudson.model;
 
+import com.infradna.tool.bridge_method_injector.WithBridgeMethods;
 import hudson.Functions;
-import java.util.regex.Pattern;
 import antlr.ANTLRException;
 import hudson.AbortException;
 import hudson.CopyOnWrite;
@@ -41,11 +41,13 @@ import hudson.cli.declarative.CLIResolver;
 import hudson.diagnosis.OldDataMonitor;
 import hudson.model.Cause.LegacyCodeCause;
 import hudson.model.Cause.RemoteCause;
-import hudson.model.Cause.UserCause;
+import hudson.model.Cause.UserIdCause;
 import hudson.model.Descriptor.FormException;
 import hudson.model.Fingerprint.RangeSet;
 import hudson.model.Queue.Executable;
 import hudson.model.Queue.Task;
+import hudson.model.listeners.SCMListener;
+import hudson.model.queue.QueueTaskFuture;
 import hudson.model.queue.SubTask;
 import hudson.model.Queue.WaitingItem;
 import hudson.model.RunMap.Constructor;
@@ -81,6 +83,8 @@ import hudson.widgets.BuildHistoryWidget;
 import hudson.widgets.HistoryWidget;
 import jenkins.model.Jenkins;
 import net.sf.json.JSONObject;
+import org.kohsuke.accmod.Restricted;
+import org.kohsuke.accmod.restrictions.NoExternalUse;
 import org.kohsuke.args4j.Argument;
 import org.kohsuke.args4j.CmdLineException;
 import org.kohsuke.stapler.ForwardToView;
@@ -91,6 +95,7 @@ import org.kohsuke.stapler.QueryParameter;
 import org.kohsuke.stapler.StaplerRequest;
 import org.kohsuke.stapler.StaplerResponse;
 import org.kohsuke.stapler.export.Exported;
+import org.kohsuke.stapler.interceptor.RequirePOST;
 
 import javax.servlet.ServletException;
 import java.io.File;
@@ -124,6 +129,7 @@ import static javax.servlet.http.HttpServletResponse.*;
  * @author Kohsuke Kawaguchi
  * @see AbstractBuild
  */
+@SuppressWarnings("rawtypes")
 public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends AbstractBuild<P,R>> extends Job<P,R> implements BuildableItem {
 
     /**
@@ -140,7 +146,11 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
 
     /**
      * All the builds keyed by their build number.
+     *
+     * External code should use {@link #getBuildByNumber(int)} or {@link #getLastBuild()} and traverse via
+     * {@link Run#getPreviousBuild()}
      */
+    @Restricted(NoExternalUse.class)
     protected transient /*almost final*/ RunMap<R> builds = new RunMap<R>();
 
     /**
@@ -250,7 +260,8 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
     public void onLoad(ItemGroup<? extends Item> parent, String name) throws IOException {
         super.onLoad(parent, name);
 
-        this.builds = new RunMap<R>();
+        if (this.builds==null)
+            this.builds = new RunMap<R>();
         this.builds.load(this,new Constructor<R>() {
             public R create(File dir) throws IOException {
                 return loadBuild(dir);
@@ -292,7 +303,7 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
      */
     @Exported
     public boolean isConcurrentBuild() {
-        return Jenkins.CONCURRENT_BUILD && concurrentBuild;
+        return concurrentBuild;
     }
 
     public void setConcurrentBuild(boolean b) throws IOException {
@@ -313,6 +324,21 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
         return Jenkins.getInstance().getLabel(assignedNode);
     }
 
+    /**
+     * Set of labels relevant to this job.
+     *
+     * This method is used to determine what slaves are relevant to jobs, for example by {@link View}s.
+     * It does not affect the scheduling. This information is informational and the best-effort basis.
+     *
+     * @since 1.456
+     * @return
+     *      Minimally it should contain {@link #getAssignedLabel()}. The set can contain null element
+     *      to correspond to the null return value from {@link #getAssignedLabel()}.
+     */
+    public Set<Label> getRelevantLabels() {
+        return Collections.singleton(getAssignedLabel());
+    }
+
     /**
      * Gets the textual representation of the assigned label as it was entered by the user.
      */
@@ -768,7 +794,8 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
      * @param actions
      *      For the convenience of the caller, this array can contain null, and those will be silently ignored.
      */
-    public Future<R> scheduleBuild2(int quietPeriod, Cause c, Action... actions) {
+    @WithBridgeMethods(Future.class)
+    public QueueTaskFuture<R> scheduleBuild2(int quietPeriod, Cause c, Action... actions) {
         return scheduleBuild2(quietPeriod,c,Arrays.asList(actions));
     }
 
@@ -780,7 +807,9 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
      *      For the convenience of the caller, this collection can contain null, and those will be silently ignored.
      * @since 1.383
      */
-    public Future<R> scheduleBuild2(int quietPeriod, Cause c, Collection<? extends Action> actions) {
+    @SuppressWarnings("unchecked")
+    @WithBridgeMethods(Future.class)
+    public QueueTaskFuture<R> scheduleBuild2(int quietPeriod, Cause c, Collection<? extends Action> actions) {
         if (!isBuildable())
             return null;
 
@@ -795,7 +824,7 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
 
         WaitingItem i = Jenkins.getInstance().getQueue().schedule(this, quietPeriod, queueActions);
         if(i!=null)
-            return (Future)i.getFuture();
+            return (QueueTaskFuture)i.getFuture();
         return null;
     }
 
@@ -829,7 +858,9 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
      * Production code shouldn't be using this, but for tests this is very convenient, so this isn't marked
      * as deprecated.
      */
-    public Future<R> scheduleBuild2(int quietPeriod) {
+    @SuppressWarnings("deprecation")
+    @WithBridgeMethods(Future.class)
+    public QueueTaskFuture<R> scheduleBuild2(int quietPeriod) {
         return scheduleBuild2(quietPeriod, new LegacyCodeCause());
     }
     
@@ -837,7 +868,8 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
      * Schedules a build of this project, and returns a {@link Future} object
      * to wait for the completion of the build.
      */
-    public Future<R> scheduleBuild2(int quietPeriod, Cause c) {
+    @WithBridgeMethods(Future.class)
+    public QueueTaskFuture<R> scheduleBuild2(int quietPeriod, Cause c) {
         return scheduleBuild2(quietPeriod, c, new Action[0]);
     }
 
@@ -1070,7 +1102,8 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
     }
 
     public CauseOfBlockage getCauseOfBlockage() {
-        if (isBuilding() && !isConcurrentBuild())
+        // Block builds until they are done with post-production
+        if (isLogUpdated() && !isConcurrentBuild())
             return new BecauseOfBuildInProgress(getLastBuild());
         if (blockBuildWhenDownstreamBuilding()) {
             AbstractProject<?,?> bup = getBuildingDownstream();
@@ -1092,10 +1125,10 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
      * This means eventually there will be an automatic triggering of
      * the given project (provided that all builds went smoothly.)
      */
-    protected AbstractProject getBuildingDownstream() {
+    public AbstractProject getBuildingDownstream() {
         Set<Task> unblockedTasks = Jenkins.getInstance().getQueue().getUnblockedTasks();
 
-        for (AbstractProject tup : Jenkins.getInstance().getDependencyGraph().getTransitiveDownstream(this)) {
+        for (AbstractProject tup : getTransitiveDownstreamProjects()) {
 			if (tup!=this && (tup.isBuilding() || unblockedTasks.contains(tup)))
                 return tup;
         }
@@ -1109,10 +1142,10 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
      * This means eventually there will be an automatic triggering of
      * the given project (provided that all builds went smoothly.)
      */
-    protected AbstractProject getBuildingUpstream() {
+    public AbstractProject getBuildingUpstream() {
         Set<Task> unblockedTasks = Jenkins.getInstance().getQueue().getUnblockedTasks();
 
-        for (AbstractProject tup : Jenkins.getInstance().getDependencyGraph().getTransitiveUpstream(this)) {
+        for (AbstractProject tup : getTransitiveUpstreamProjects()) {
 			if (tup!=this && (tup.isBuilding() || unblockedTasks.contains(tup)))
                 return tup;
         }
@@ -1191,7 +1224,11 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
         workspace.mkdirs();
         
         boolean r = scm.checkout(build, launcher, workspace, listener, changelogFile);
-        calcPollingBaseline(build, launcher, listener);
+        if (r) {
+            // Only calcRevisionsFromBuild if checkout was successful. Note that modern SCM implementations
+            // won't reach this line anyway, as they throw AbortExceptions on checkout failure.
+            calcPollingBaseline(build, launcher, listener);
+        }
         return r;
     }
 
@@ -1212,13 +1249,6 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
         pollingBaseline = baseline;
     }
 
-    /**
-     * For reasons I don't understand, if I inline this method, AbstractMethodError escapes try/catch block.
-     */
-    private SCMRevisionState safeCalcRevisionsFromBuild(AbstractBuild build, Launcher launcher, TaskListener listener) throws IOException, InterruptedException {
-        return getScm()._calcRevisionsFromBuild(build, launcher, listener);
-    }
-
     /**
      * Checks if there's any update in SCM, and returns true if any is found.
      *
@@ -1419,6 +1449,7 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
         }
     }
 
+    @SuppressWarnings("unchecked")
     public synchronized Map<TriggerDescriptor,Trigger> getTriggers() {
         return (Map)Descriptor.toMap(triggers);
     }
@@ -1589,7 +1620,7 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
             String causeText = req.getParameter("cause");
             cause = new RemoteCause(req.getRemoteAddr(), causeText);
         } else {
-            cause = new UserCause();
+            cause = new UserIdCause();
         }
         return new CauseAction(cause);
     }
@@ -1640,12 +1671,28 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
      * Cancels a scheduled build.
      */
     public void doCancelQueue( StaplerRequest req, StaplerResponse rsp ) throws IOException, ServletException {
-        checkPermission(BUILD);
+        checkPermission(ABORT);
 
         Jenkins.getInstance().getQueue().cancel(this);
         rsp.forwardToPreviousPage(req);
     }
 
+    /**
+     * Deletes this project.
+     */
+    @Override
+    @RequirePOST
+    public void doDoDelete(StaplerRequest req, StaplerResponse rsp) throws IOException, ServletException, InterruptedException {
+        delete();
+        if (req == null || rsp == null)
+            return;
+        View view = req.findAncestorObject(View.class);
+        if (view == null)
+            rsp.sendRedirect2(req.getContextPath() + '/' + getParent().getUrl());
+        else 
+            rsp.sendRedirect2(req.getContextPath() + '/' + view.getUrl());
+    }
+    
     @Override
     protected void submit(StaplerRequest req, StaplerResponse rsp) throws IOException, ServletException, FormException {
         super.submit(req,rsp);
@@ -1745,6 +1792,9 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
         FilePath ws = b!=null ? b.getWorkspace() : null;
         if (ws!=null && getScm().processWorkspaceBeforeDeletion(this, ws, b.getBuiltOn())) {
             ws.deleteRecursive();
+            for (WorkspaceListener wl : WorkspaceListener.all()) {
+                wl.afterDelete(this);
+            }
             return new HttpRedirect(".");
         } else {
             // If we get here, that means the SCM blocked the workspace deletion.
@@ -1753,16 +1803,16 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
     }
 
     @CLIMethod(name="disable-job")
+    @RequirePOST
     public HttpResponse doDisable() throws IOException, ServletException {
-        requirePOST();
         checkPermission(CONFIGURE);
         makeDisabled(true);
         return new HttpRedirect(".");
     }
 
     @CLIMethod(name="enable-job")
+    @RequirePOST
     public HttpResponse doEnable() throws IOException, ServletException {
-        requirePOST();
         checkPermission(CONFIGURE);
         makeDisabled(false);
         return new HttpRedirect(".");
@@ -1881,7 +1931,7 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
             return FormValidation.ok();
         }
 
-        public FormValidation doCheckCustomWorkspace(@QueryParameter String customWorkspace){
+        public FormValidation doCheckCustomWorkspace(@QueryParameter(value="customWorkspace.directory") String customWorkspace){
         	if(Util.fixEmptyAndTrim(customWorkspace)==null)
         		return FormValidation.error("Custom workspace is empty");
         	else
@@ -1922,14 +1972,13 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
          */
         static class AutoCompleteSeeder {
             private String source;
-            private Pattern quoteMatcher = Pattern.compile("(\\\"?)(.+?)(\\\"?+)(\\s*)");
 
             AutoCompleteSeeder(String source) {
                 this.source = source;
             }
 
             List<String> getSeeds() {
-                ArrayList<String> terms = new ArrayList();
+                ArrayList<String> terms = new ArrayList<String>();
                 boolean trailingQuote = source.endsWith("\"");
                 boolean leadingQuote = source.startsWith("\"");
                 boolean trailingSpace = source.endsWith(" ");
@@ -1990,9 +2039,9 @@ public abstract class AbstractProject<P extends AbstractProject<P,R>,R extends A
     private static final Logger LOGGER = Logger.getLogger(AbstractProject.class.getName());
 
     /**
-     * Permission to abort a build. For now, let's make it the same as {@link #BUILD}
+     * Permission to abort a build
      */
-    public static final Permission ABORT = BUILD;
+    public static final Permission ABORT = CANCEL;
 
     /**
      * Replaceable "Build Now" text.
