@@ -25,6 +25,7 @@ package hudson.model;
 
 import hudson.BulkChange;
 import hudson.DescriptorExtensionList;
+import hudson.Extension;
 import hudson.Util;
 import hudson.XmlFile;
 import hudson.model.Descriptor.FormException;
@@ -39,6 +40,7 @@ import org.kohsuke.stapler.StaplerRequest;
 import org.kohsuke.stapler.StaplerResponse;
 import org.kohsuke.stapler.export.Exported;
 import org.kohsuke.stapler.export.ExportedBean;
+import org.kohsuke.stapler.interceptor.RequirePOST;
 
 import javax.servlet.ServletException;
 import static javax.servlet.http.HttpServletResponse.SC_BAD_REQUEST;
@@ -59,7 +61,7 @@ import java.util.logging.Logger;
  * @author Kohsuke Kawaguchi
  */
 @ExportedBean
-public final class ComputerSet extends AbstractModelObject {
+public final class ComputerSet extends AbstractModelObject implements Describable<ComputerSet> {
     /**
      * This is the owner that persists {@link #monitors}.
      */
@@ -298,10 +300,11 @@ public final class ComputerSet extends AbstractModelObject {
             return FormValidation.error(e.getMessage());
         }
     }
-
+    
     /**
      * Accepts submission from the configuration page.
      */
+    @RequirePOST
     public synchronized void doConfigSubmit( StaplerRequest req, StaplerResponse rsp ) throws IOException, ServletException, FormException {
         BulkChange bc = new BulkChange(MONITORS_OWNER);
         try {
@@ -332,6 +335,32 @@ public final class ComputerSet extends AbstractModelObject {
         return new Api(this);
     }
 
+    public Descriptor<ComputerSet> getDescriptor() {
+        return Jenkins.getInstance().getDescriptorOrDie(ComputerSet.class);
+    }
+
+    @Extension
+    public static class DescriptorImpl extends Descriptor<ComputerSet> {
+        @Override
+        public String getDisplayName() {
+            return "";
+        }
+
+        /**
+         * Auto-completion for the "copy from" field in the new job page.
+         */
+        public AutoCompletionCandidates doAutoCompleteCopyNewItemFrom(@QueryParameter final String value) {
+            final AutoCompletionCandidates r = new AutoCompletionCandidates();
+
+            for (Node n : Jenkins.getInstance().getNodes()) {
+                if (n.getNodeName().startsWith(value))
+                    r.add(n.getNodeName());
+            }
+
+            return r;
+        }
+    }
+
     /**
      * Just to force the execution of the static initializer.
      */
