@@ -84,12 +84,9 @@ public class DependencyGraph implements Comparator<AbstractProject> {
     public void build() {
         // Set full privileges while computing to avoid missing any projects the current user cannot see.
         // Use setContext (NOT getContext().setAuthentication()) so we don't affect concurrent threads for same HttpSession.
-        SecurityContext saveCtx = SecurityContextHolder.getContext();
+        SecurityContext saveCtx = ACL.impersonate(ACL.SYSTEM);
         try {
             this.computationalData = new HashMap<Class<?>, Object>();
-            NotSerilizableSecurityContext system = new NotSerilizableSecurityContext();
-            system.setAuthentication(ACL.SYSTEM);
-            SecurityContextHolder.setContext(system);
             for( AbstractProject p : getAllProjects() )
                 p.buildDependencyGraph(this);
 
