@@ -23,6 +23,8 @@
  */
 package hudson.model;
 
+import com.google.common.base.Function;
+import com.google.common.collect.Collections2;
 import com.infradna.tool.bridge_method_injector.WithBridgeMethods;
 import hudson.Extension;
 import hudson.ExtensionPoint;
@@ -33,6 +35,7 @@ import hudson.model.Descriptor.FormException;
 import hudson.model.Fingerprint.Range;
 import hudson.model.Fingerprint.RangeSet;
 import hudson.model.PermalinkProjectAction.Permalink;
+import hudson.model.listeners.ItemListener;
 import hudson.search.QuickSilver;
 import hudson.search.SearchIndex;
 import hudson.search.SearchIndexBuilder;
@@ -46,8 +49,8 @@ import hudson.util.ColorPalette;
 import hudson.util.CopyOnWriteList;
 import hudson.util.DataSetBuilder;
 import hudson.util.DescribableList;
+import hudson.util.FormApply;
 import hudson.util.Graph;
-import hudson.util.IOException2;
 import hudson.util.RunList;
 import hudson.util.ShiftedCategoryAxis;
 import hudson.util.StackedAreaRenderer2;
@@ -56,6 +59,7 @@ import hudson.widgets.HistoryWidget;
 import hudson.widgets.HistoryWidget.Adapter;
 import hudson.widgets.Widget;
 import jenkins.model.Jenkins;
+import jenkins.model.ProjectNamingStrategy;
 import net.sf.json.JSONException;
 import net.sf.json.JSONObject;
 import org.jfree.chart.ChartFactory;
@@ -75,21 +79,14 @@ import org.kohsuke.stapler.StaplerOverridable;
 import org.kohsuke.stapler.StaplerRequest;
 import org.kohsuke.stapler.StaplerResponse;
 import org.kohsuke.stapler.export.Exported;
+import org.kohsuke.stapler.interceptor.RequirePOST;
 
 import javax.servlet.ServletException;
 import java.awt.*;
-import java.io.File;
-import java.io.IOException;
-import java.io.PrintWriter;
-import java.io.StringWriter;
+import java.io.*;
 import java.net.URLEncoder;
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.Collections;
-import java.util.LinkedList;
+import java.util.*;
 import java.util.List;
-import java.util.Map;
-import java.util.SortedMap;
 
 import static javax.servlet.http.HttpServletResponse.*;
 
@@ -166,7 +163,24 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
                     this.nextBuildNumber = Integer.parseInt(f.readTrim());
                 }
             } catch (NumberFormatException e) {
-                throw new IOException2(f + " doesn't contain a number", e);
+                // try to infer the value of the next build number from the existing build records. See JENKINS-11563
+                File[] folders = this.getBuildDir().listFiles(new FileFilter() {
+                    public boolean accept(File file) {
+                        return file.isDirectory() && file.getName().matches("[0-9]+");
+                    }
+                });
+
+                if (folders == null || folders.length == 0) {
+                    this.nextBuildNumber = 1;
+                } else {
+                    Collection<Integer> foldersInt = Collections2.transform(Arrays.asList(folders), new Function<File, Integer>() {
+                        public Integer apply(File file) {
+                            return Integer.parseInt(file.getName());
+                        }
+                    });
+                    this.nextBuildNumber = Collections.max(foldersInt) + 1;
+                }
+                saveNextBuildNumber();
             }
         } else {
             // From the old Hudson, or doCreateItem. Create this file now.
@@ -239,6 +253,14 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
         RunT b = getLastBuild();
         return b!=null && b.isBuilding();
     }
+    
+    /**
+     * Returns true if the log file is still being updated.
+     */
+    public boolean isLogUpdated() {
+        RunT b = getLastBuild();
+        return b!=null && b.isLogUpdated();
+    }    
 
     @Override
     public String getPronoun() {
@@ -782,7 +804,7 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
         return result;
     }
     
-    public final long getEstimatedDuration() {
+    public long getEstimatedDuration() {
         List<RunT> builds = getLastBuildsOverThreshold(3, Result.UNSTABLE);
         
         if(builds.isEmpty())     return -1;
@@ -938,6 +960,7 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
     /**
      * Accepts submission from the configuration page.
      */
+    @RequirePOST
     public synchronized void doConfigSubmit(StaplerRequest req,
             StaplerResponse rsp) throws IOException, ServletException, FormException {
         checkPermission(CONFIGURE);
@@ -949,6 +972,8 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
         try {
             JSONObject json = req.getSubmittedForm();
 
+            setDisplayName(json.optString("displayNameOrNull"));
+
             if (req.getParameter("logrotate") != null)
                 logRotator = LogRotator.DESCRIPTOR.newInstance(req,json.getJSONObject("logrotate"));
             else
@@ -965,14 +990,20 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
             submit(req, rsp);
 
             save();
+            ItemListener.fireOnUpdated(this);
 
             String newName = req.getParameter("name");
+            final ProjectNamingStrategy namingStrategy = Jenkins.getInstance().getProjectNamingStrategy();
             if (newName != null && !newName.equals(name)) {
                 // check this error early to avoid HTTP response splitting.
                 Jenkins.checkGoodName(newName);
+                namingStrategy.checkName(newName);
                 rsp.sendRedirect("rename?newName=" + URLEncoder.encode(newName, "UTF-8"));
             } else {
-                rsp.sendRedirect(".");
+                if(namingStrategy.isForceExistingJobs()){
+                    namingStrategy.checkName(name);
+                }
+                FormApply.success(".").generateResponse(req, rsp, null);
             }
         } catch (JSONException e) {
             StringWriter sw = new StringWriter();
@@ -1171,10 +1202,10 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
     /**
      * Renames this job.
      */
+    @RequirePOST
     public/* not synchronized. see renameTo() */void doDoRename(
             StaplerRequest req, StaplerResponse rsp) throws IOException,
             ServletException {
-        requirePOST();
         // rename is essentially delete followed by a create
         checkPermission(CREATE);
         checkPermission(DELETE);
@@ -1192,8 +1223,7 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
         // send to the new job page
         // note we can't use getUrl() because that would pick up old name in the
         // Ancestor.getUrl()
-        rsp.sendRedirect2(req.getContextPath() + '/' + getParent().getUrl()
-                + getShortUrl());
+        rsp.sendRedirect2("../" + newName);
     }
 
     public void doRssAll(StaplerRequest req, StaplerResponse rsp)
