@@ -36,6 +36,7 @@ import static hudson.util.Iterators.reverse;
 
 import hudson.cli.declarative.CLIMethod;
 import hudson.cli.declarative.CLIResolver;
+import hudson.model.labels.LabelAssignmentAction;
 import hudson.model.queue.AbstractQueueTask;
 import hudson.model.queue.Executables;
 import hudson.model.queue.SubTask;
@@ -78,7 +79,6 @@ import java.util.GregorianCalendar;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
-import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.NoSuchElementException;
@@ -93,6 +93,7 @@ import java.util.logging.Logger;
 import javax.management.timer.Timer;
 import javax.servlet.ServletException;
 
+import jenkins.model.Jenkins;
 import org.acegisecurity.AccessDeniedException;
 import org.kohsuke.stapler.HttpResponse;
 import org.kohsuke.stapler.HttpResponses;
@@ -140,8 +141,9 @@ public class Queue extends ResourceController implements Saveable {
     /**
      * {@link Task}s that can be built immediately
      * but blocked because another build is in progress,
-     * required {@link Resource}s are not available, or otherwise blocked
-     * by {@link Task#isBuildBlocked()}.
+     * required {@link Resource}s are not available,
+     * blocked via {@link QueueTaskDispatcher#canRun(Item)},
+     * or otherwise blocked by {@link Task#isBuildBlocked()}.
      */
     private final ItemList<BlockedItem> blockedProjects = new ItemList<BlockedItem>();
 
@@ -201,15 +203,15 @@ public class Queue extends ResourceController implements Saveable {
         /**
          * Verifies that the {@link Executor} represented by this object is capable of executing the given task.
          */
-        public boolean canTake(Task task) {
+        public boolean canTake(BuildableItem item) {
             Node node = getNode();
             if (node==null)     return false;   // this executor is about to die
 
-            if(node.canTake(task)!=null)
+            if(node.canTake(item)!=null)
                 return false;   // this node is not able to take the task
 
             for (QueueTaskDispatcher d : QueueTaskDispatcher.all())
-                if (d.canTake(node,task)!=null)
+                if (d.canTake(node,item)!=null)
                     return false;
 
             return isAvailable();
@@ -253,7 +255,7 @@ public class Queue extends ResourceController implements Saveable {
 
     public void setLoadBalancer(LoadBalancer loadBalancer) {
         if(loadBalancer==null)  throw new IllegalArgumentException();
-        this.loadBalancer = loadBalancer;
+        this.loadBalancer = loadBalancer.sanitize();
     }
 
     public QueueSorter getSorter() {
@@ -273,13 +275,16 @@ public class Queue extends ResourceController implements Saveable {
             File queueFile = getQueueFile();
             if (queueFile.exists()) {
                 BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(queueFile)));
-                String line;
-                while ((line = in.readLine()) != null) {
-                    AbstractProject j = Hudson.getInstance().getItemByFullName(line, AbstractProject.class);
-                    if (j != null)
-                        j.scheduleBuild();
+                try {
+                    String line;
+                    while ((line = in.readLine()) != null) {
+                        AbstractProject j = Jenkins.getInstance().getItemByFullName(line, AbstractProject.class);
+                        if (j != null)
+                            j.scheduleBuild();
+                    }
+                } finally {
+                    in.close();
                 }
-                in.close();
                 // discard the queue file now that we are done
                 queueFile.delete();
             } else {
@@ -353,7 +358,7 @@ public class Queue extends ResourceController implements Saveable {
      */
     @CLIMethod(name="clear-queue")
     public synchronized void clear() {
-        Hudson.getInstance().checkPermission(Hudson.ADMINISTER);
+        Jenkins.getInstance().checkPermission(Jenkins.ADMINISTER);
         for (WaitingItem i : waitingList)
             i.onCancelled();
         waitingList.clear();
@@ -363,11 +368,11 @@ public class Queue extends ResourceController implements Saveable {
     }
 
     private File getQueueFile() {
-        return new File(Hudson.getInstance().getRootDir(), "queue.txt");
+        return new File(Jenkins.getInstance().getRootDir(), "queue.txt");
     }
 
     /*package*/ File getXMLQueueFile() {
-        return new File(Hudson.getInstance().getRootDir(), "queue.xml");
+        return new File(Jenkins.getInstance().getRootDir(), "queue.xml");
     }
 
     /**
@@ -614,7 +619,7 @@ public class Queue extends ResourceController implements Saveable {
     private void _getBuildableItems(Computer c, ItemList<BuildableItem> col, List<BuildableItem> result) {
         Node node = c.getNode();
         for (BuildableItem p : col.values()) {
-            if (node.canTake(p.task) == null)
+            if (node.canTake(p) == null)
                 result.add(p);
         }
     }
@@ -678,10 +683,10 @@ public class Queue extends ResourceController implements Saveable {
     public synchronized int countBuildableItemsFor(Label l) {
         int r = 0;
         for (BuildableItem bi : buildables.values())
-            if(bi.task.getAssignedLabel()==l)
+            if(bi.getAssignedLabel()==l)
                 r++;
         for (BuildableItem bi : pendings.values())
-            if(bi.task.getAssignedLabel()==l)
+            if(bi.getAssignedLabel()==l)
                 r++;
         return r;
     }
@@ -777,40 +782,6 @@ public class Queue extends ResourceController implements Saveable {
                 // in the buildables field.
                 maintain();
 
-                // allocate buildable jobs to executors
-                Iterator<BuildableItem> itr = buildables.iterator();
-                while (itr.hasNext()) {
-                    BuildableItem p = itr.next();
-
-                    // one last check to make sure this build is not blocked.
-                    if (isBuildBlocked(p.task)) {
-                        itr.remove();
-                        blockedProjects.put(p.task,new BlockedItem(p));
-                        continue;
-                    }
-
-                    List<JobOffer> candidates = new ArrayList<JobOffer>(parked.size());
-                    for (JobOffer j : parked.values())
-                        if(j.canTake(p.task))
-                            candidates.add(j);
-
-                    MappingWorksheet ws = new MappingWorksheet(p, candidates);
-                    Mapping m = loadBalancer.map(p.task, ws);
-                    if (m == null)
-                        // if we couldn't find the executor that fits,
-                        // just leave it in the buildables list and
-                        // check if we can execute other projects
-                        continue;
-
-                    // found a matching executor. use it.
-                    WorkUnitContext wuc = new WorkUnitContext(p);
-                    m.execute(wuc);
-
-                    itr.remove();
-                    if (!wuc.getWorkUnits().isEmpty())
-                        pendings.add(p);
-                }
-
                 // we went over all the buildable projects and awaken
                 // all the executors that got work to do. now, go to sleep
                 // until this thread is awakened. If this executor assigned a job to
@@ -881,10 +852,19 @@ public class Queue extends ResourceController implements Saveable {
     }
 
     /**
-     * Checks if the given task is blocked.
+     * Checks if the given item should be prevented from entering into the {@link #buildables} state
+     * and instead stay in the {@link #blockedProjects} state.
      */
-    private boolean isBuildBlocked(Task t) {
-        return t.isBuildBlocked() || !canRun(t.getResourceList());
+    private boolean isBuildBlocked(Item i) {
+        if (i.task.isBuildBlocked() || !canRun(i.task.getResourceList()))
+            return true;
+
+        for (QueueTaskDispatcher d : QueueTaskDispatcher.all()) {
+            if (d.canRun(i)!=null)
+                return true;
+        }
+
+        return false;
     }
 
     /**
@@ -903,35 +883,43 @@ public class Queue extends ResourceController implements Saveable {
 
     /**
      * Queue maintenance.
+     *
      * <p>
-     * Move projects between {@link #waitingList}, {@link #blockedProjects}, and {@link #buildables}
+     * Move projects between {@link #waitingList}, {@link #blockedProjects}, {@link #buildables}, and {@link #pendings}
      * appropriately.
+     *
+     * <p>
+     * Jenkins internally invokes this method by itself whenever there's a change that can affect
+     * the scheduling (such as new node becoming online, # of executors change, a task completes execution, etc.),
+     * and it also gets invoked periodically (see {@link MaintainTask}.)
      */
     public synchronized void maintain() {
         if (LOGGER.isLoggable(Level.FINE))
             LOGGER.fine("Queue maintenance started " + this);
 
-        // blocked -> buildable
-        Iterator<BlockedItem> itr = blockedProjects.values().iterator();
-        while (itr.hasNext()) {
-            BlockedItem p = itr.next();
-            if (!isBuildBlocked(p.task) && allowNewBuildableTask(p.task)) {
-                // ready to be executed
-                LOGGER.fine(p.task.getFullDisplayName() + " no longer blocked");
-                itr.remove();
-                makeBuildable(new BuildableItem(p));
+        {// blocked -> buildable
+            Iterator<BlockedItem> itr = blockedProjects.values().iterator();
+            while (itr.hasNext()) {
+                BlockedItem p = itr.next();
+                if (!isBuildBlocked(p) && allowNewBuildableTask(p.task)) {
+                    // ready to be executed
+                    LOGGER.fine(p.task.getFullDisplayName() + " no longer blocked");
+                    itr.remove();
+                    makeBuildable(new BuildableItem(p));
+                }
             }
         }
 
+        // waitingList -> buildable/blocked
         while (!waitingList.isEmpty()) {
             WaitingItem top = peek();
 
             if (!top.timestamp.before(new GregorianCalendar()))
-                return; // finished moving all ready items from queue
+                break; // finished moving all ready items from queue
 
             waitingList.remove(top);
             Task p = top.task;
-            if (!isBuildBlocked(p) && allowNewBuildableTask(p)) {
+            if (!isBuildBlocked(top) && allowNewBuildableTask(p)) {
                 // ready to be executed immediately
                 LOGGER.fine(p.getFullDisplayName() + " ready to build");
                 makeBuildable(new BuildableItem(top));
@@ -946,25 +934,60 @@ public class Queue extends ResourceController implements Saveable {
         final QueueSorter s = sorter;
         if (s != null)
         	s.sortBuildableItems(buildables);
+
+        // allocate buildable jobs to executors
+        Iterator<BuildableItem> itr = buildables.iterator();
+        while (itr.hasNext()) {
+            BuildableItem p = itr.next();
+
+            // one last check to make sure this build is not blocked.
+            if (isBuildBlocked(p)) {
+                itr.remove();
+                blockedProjects.put(p.task,new BlockedItem(p));
+                continue;
+            }
+
+            List<JobOffer> candidates = new ArrayList<JobOffer>(parked.size());
+            for (JobOffer j : parked.values())
+                if(j.canTake(p))
+                    candidates.add(j);
+
+            MappingWorksheet ws = new MappingWorksheet(p, candidates);
+            Mapping m = loadBalancer.map(p.task, ws);
+            if (m == null)
+                // if we couldn't find the executor that fits,
+                // just leave it in the buildables list and
+                // check if we can execute other projects
+                continue;
+
+            // found a matching executor. use it.
+            WorkUnitContext wuc = new WorkUnitContext(p);
+            m.execute(wuc);
+
+            itr.remove();
+            if (!wuc.getWorkUnits().isEmpty())
+                pendings.add(p);
+        }
     }
 
     private void makeBuildable(BuildableItem p) {
-        if(Hudson.FLYWEIGHT_SUPPORT && p.task instanceof FlyweightTask && !ifBlockedByHudsonShutdown(p.task)) {
+        if(Jenkins.FLYWEIGHT_SUPPORT && p.task instanceof FlyweightTask && !ifBlockedByHudsonShutdown(p.task)) {
             ConsistentHash<Node> hash = new ConsistentHash<Node>(new Hash<Node>() {
                 public String hash(Node node) {
                     return node.getNodeName();
                 }
             });
-            Hudson h = Hudson.getInstance();
+            Jenkins h = Jenkins.getInstance();
             hash.add(h, h.getNumExecutors()*100);
             for (Node n : h.getNodes())
                 hash.add(n,n.getNumExecutors()*100);
 
-            Label lbl = p.task.getAssignedLabel();
+            Label lbl = p.getAssignedLabel();
             for (Node n : hash.list(p.task.getFullDisplayName())) {
                 Computer c = n.toComputer();
                 if (c==null || c.isOffline())    continue;
                 if (lbl!=null && !lbl.contains(n))  continue;
+                if (n.canTake(p) != null) continue;
                 c.startFlyWeightTask(new WorkUnitContext(p).createWorkUnit(p.task));
                 pendings.add(p);
                 return;
@@ -977,7 +1000,7 @@ public class Queue extends ResourceController implements Saveable {
     }
 
     public static boolean ifBlockedByHudsonShutdown(Task task) {
-        return Hudson.getInstance().isQuietingDown() && !(task instanceof NonBlockingTask);
+        return Jenkins.getInstance().isQuietingDown() && !(task instanceof NonBlockingTask);
     }
 
     public Api getApi() {
@@ -998,7 +1021,7 @@ public class Queue extends ResourceController implements Saveable {
     public interface FlyweightTask extends Task {}
 
     /**
-     * Marks {@link Task}s that are not affected by the {@linkplain Hudson#isQuietingDown()}  quieting down},
+     * Marks {@link Task}s that are not affected by the {@linkplain Jenkins#isQuietingDown()}  quieting down},
      * because these tasks keep other tasks executing.
      *
      * @since 1.336 
@@ -1098,7 +1121,8 @@ public class Queue extends ResourceController implements Saveable {
         String getUrl();
         
         /**
-         * True if the task allows concurrent builds
+         * True if the task allows concurrent builds, where the same {@link Task} is executed
+         * by multiple executors concurrently on the same or different nodes.
          *
          * @since 1.338
          */
@@ -1176,6 +1200,7 @@ public class Queue extends ResourceController implements Saveable {
          * VM-wide unique ID that tracks the {@link Task} as it moves through different stages
          * in the queue (each represented by different subtypes of {@link Item}.
          */
+        @Exported
     	public final int id;
     	
 		/**
@@ -1185,6 +1210,8 @@ public class Queue extends ResourceController implements Saveable {
         public final Task task;
 
         private /*almost final*/ transient FutureImpl future;
+        
+        private final long inQueueSince;
 
         /**
          * Build is blocked because another build is in progress,
@@ -1207,6 +1234,24 @@ public class Queue extends ResourceController implements Saveable {
          */
         @Exported
         public boolean isStuck() { return false; }
+        
+        /**
+         * Since when is this item in the queue.
+         * @return Unix timestamp
+         */
+        @Exported
+        public long getInQueueSince() {
+            return this.inQueueSince;
+        }
+        
+        /**
+         * Returns a human readable presentation of how long this item is already in the queue.
+         * E.g. something like '3 minutes 40 seconds'
+         */
+        public String getInQueueForString() {
+            long duration = System.currentTimeMillis() - this.inQueueSince;
+            return Util.getTimeSpanString(duration);
+        }
 
         /**
          * Can be used to wait for the completion (either normal, abnormal, or cancellation) of the {@link Task}.
@@ -1215,6 +1260,14 @@ public class Queue extends ResourceController implements Saveable {
          */
         public Future<Executable> getFuture() { return future; }
 
+        public Label getAssignedLabel() {
+            for (LabelAssignmentAction laa : getActions(LabelAssignmentAction.class)) {
+                Label l = laa.getAssignedLabel(task);
+                if (l!=null)    return l;
+            }
+            return task.getAssignedLabel();
+        }
+
         /**
          * Convenience method that returns a read only view of the {@link Cause}s associated with this item in the queue.
          *
@@ -1232,11 +1285,20 @@ public class Queue extends ResourceController implements Saveable {
             this.task = task;
             this.id = id;
             this.future = future;
+            this.inQueueSince = System.currentTimeMillis();
+            for (Action action: actions) addAction(action);
+        }
+        
+        protected Item(Task task, List<Action> actions, int id, FutureImpl future, long inQueueSince) {
+            this.task = task;
+            this.id = id;
+            this.future = future;
+            this.inQueueSince = inQueueSince;
             for (Action action: actions) addAction(action);
         }
         
         protected Item(Item item) {
-        	this(item.task, item.getActions(), item.id, item.future);
+        	this(item.task, item.getActions(), item.id, item.future, item.inQueueSince);
         }
 
         /**
@@ -1289,7 +1351,7 @@ public class Queue extends ResourceController implements Saveable {
          * Called from queue.jelly.
          */
         public HttpResponse doCancelQueue() throws IOException, ServletException {
-        	Hudson.getInstance().getQueue().cancel(this);
+        	Jenkins.getInstance().getQueue().cancel(this);
             return HttpResponses.forwardToPreviousPage();
         }
 
@@ -1322,7 +1384,7 @@ public class Queue extends ResourceController implements Saveable {
     	 * Returns whether the new item should be scheduled. 
     	 * An action should return true if the associated task is 'different enough' to warrant a separate execution.
     	 */
-    	public boolean shouldSchedule(List<Action> actions);
+	    boolean shouldSchedule(List<Action> actions);
     }
 
     /**
@@ -1334,14 +1396,18 @@ public class Queue extends ResourceController implements Saveable {
      * to the queue, and the task will never get executed. 
      *
      * <p>
-     * This extension point is still a subject to change, as we are seeking more
-     * comprehensive Queue pluggability. See HUDSON-2072.
+     * The other use case is to add additional {@link Action}s to the task
+     * (for example {@link LabelAssignmentAction}) to tasks that are submitted to the queue.
      *
      * @since 1.316
      */
     public static abstract class QueueDecisionHandler implements ExtensionPoint {
     	/**
     	 * Returns whether the new item should be scheduled.
+         *
+         * @param actions
+         *      List of actions that are to be made available as {@link AbstractBuild#getActions()}
+         *      upon the start of the build. This list is live, and can be mutated.
     	 */
     	public abstract boolean shouldSchedule(Task p, List<Action> actions);
     	    	
@@ -1350,7 +1416,7 @@ public class Queue extends ResourceController implements Saveable {
     	 * @return
     	 */
     	public static ExtensionList<QueueDecisionHandler> all() {
-    		return Hudson.getInstance().getExtensionList(QueueDecisionHandler.class);
+    		return Jenkins.getInstance().getExtensionList(QueueDecisionHandler.class);
     	}
     }
     
@@ -1427,6 +1493,13 @@ public class Queue extends ResourceController implements Saveable {
                     return CauseOfBlockage.fromMessage(Messages._Queue_InProgress());
                 return CauseOfBlockage.fromMessage(Messages._Queue_BlockedBy(r.getDisplayName()));
             }
+            
+            for (QueueTaskDispatcher d : QueueTaskDispatcher.all()) {
+                CauseOfBlockage cause = d.canRun(this);
+                if (cause != null)
+                    return cause;
+            }
+            
             return task.getCauseOfBlockage();
         }
     }
@@ -1444,33 +1517,48 @@ public class Queue extends ResourceController implements Saveable {
         }
 
         public CauseOfBlockage getCauseOfBlockage() {
-            Hudson hudson = Hudson.getInstance();
+            Jenkins jenkins = Jenkins.getInstance();
             if(ifBlockedByHudsonShutdown(task))
                 return CauseOfBlockage.fromMessage(Messages._Queue_HudsonIsAboutToShutDown());
 
-            Label label = task.getAssignedLabel();
-            if (hudson.getNodes().isEmpty())
+            Label label = getAssignedLabel();
+            List<Node> allNodes = jenkins.getNodes();
+            if (allNodes.isEmpty())
                 label = null;    // no master/slave. pointless to talk about nodes
 
             if (label != null) {
+                Set<Node> nodes = label.getNodes();
                 if (label.isOffline()) {
-                    Set<Node> nodes = label.getNodes();
                     if (nodes.size() != 1)      return new BecauseLabelIsOffline(label);
                     else                        return new BecauseNodeIsOffline(nodes.iterator().next());
+                } else {
+                    if (nodes.size() != 1)      return new BecauseLabelIsBusy(label);
+                    else                        return new BecauseNodeIsBusy(nodes.iterator().next());
                 }
+            } else {
+                CauseOfBlockage c = null;
+                for (Node node : allNodes) {
+                    if (node.toComputer().isPartiallyIdle()) {
+                        c = canTake(node);
+                        if (c==null)    break;
+                    }
+                }
+
+                return CauseOfBlockage.createNeedsMoreExecutor(Messages._Queue_WaitingForNextAvailableExecutor());
             }
+        }
 
-            if(label==null)
-                return CauseOfBlockage.fromMessage(Messages._Queue_WaitingForNextAvailableExecutor());
-
-            Set<Node> nodes = label.getNodes();
-            if (nodes.size() != 1)      return new BecauseLabelIsBusy(label);
-            else                        return new BecauseNodeIsBusy(nodes.iterator().next());
+        private CauseOfBlockage canTake(Node node) {
+            for (QueueTaskDispatcher d : QueueTaskDispatcher.all()) {
+                CauseOfBlockage cause = d.canTake(node, this);
+                if (cause!=null)    return cause;
+            }
+            return null;
         }
 
         @Override
         public boolean isStuck() {
-            Label label = task.getAssignedLabel();
+            Label label = getAssignedLabel();
             if(label!=null && label.isOffline())
                 // no executor online to process this job. definitely stuck.
                 return true;
@@ -1505,7 +1593,7 @@ public class Queue extends ResourceController implements Saveable {
 
 			@Override
 			public Object fromString(String string) {
-                Object item = Hudson.getInstance().getItemByFullName(string);
+                Object item = Jenkins.getInstance().getItemByFullName(string);
                 if(item==null)  throw new NoSuchElementException("No such job exists: "+string);
                 return item;
 			}
@@ -1528,7 +1616,7 @@ public class Queue extends ResourceController implements Saveable {
 				String[] split = string.split("#");
 				String projectName = split[0];
 				int buildNumber = Integer.parseInt(split[1]);
-				Job<?,?> job = (Job<?,?>) Hudson.getInstance().getItemByFullName(projectName);
+				Job<?,?> job = (Job<?,?>) Jenkins.getInstance().getItemByFullName(projectName);
                 if(job==null)  throw new NoSuchElementException("No such job exists: "+projectName);
 				Run<?,?> run = job.getBuildByNumber(buildNumber);
                 if(run==null)  throw new NoSuchElementException("No such build: "+string);
@@ -1641,14 +1729,14 @@ public class Queue extends ResourceController implements Saveable {
 
     @CLIResolver
     public static Queue getInstance() {
-        return Hudson.getInstance().getQueue();
+        return Jenkins.getInstance().getQueue();
     }
 
     /**
      * Restores the queue content during the start up.
      */
     @Initializer(after=JOB_LOADED)
-    public static void init(Hudson h) {
+    public static void init(Jenkins h) {
         h.getQueue().load();
     }
 }
