@@ -45,10 +45,13 @@ import hudson.util.HttpResponses;
 import hudson.util.IOException2;
 import hudson.util.PersistedList;
 import hudson.util.XStream2;
+import jenkins.RestartRequiredException;
 import jenkins.model.Jenkins;
 import org.acegisecurity.Authentication;
+import org.acegisecurity.context.SecurityContext;
 import org.apache.commons.io.input.CountingInputStream;
 import org.apache.commons.io.output.NullOutputStream;
+import org.jvnet.localizer.Localizable;
 import org.kohsuke.stapler.HttpResponse;
 import org.kohsuke.stapler.StaplerRequest;
 import org.kohsuke.stapler.StaplerResponse;
@@ -80,6 +83,9 @@ import java.util.jar.JarFile;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 import org.acegisecurity.context.SecurityContextHolder;
+import org.kohsuke.stapler.export.Exported;
+import org.kohsuke.stapler.export.ExportedBean;
+import org.kohsuke.stapler.interceptor.RequirePOST;
 
 
 /**
@@ -97,7 +103,11 @@ import org.acegisecurity.context.SecurityContextHolder;
  * @author Kohsuke Kawaguchi
  * @since 1.220
  */
+@ExportedBean
 public class UpdateCenter extends AbstractModelObject implements Saveable {
+	
+    private static final String UPDATE_CENTER_URL = System.getProperty(UpdateCenter.class.getName()+".updateCenterUrl","http://updates.jenkins-ci.org/");
+	
     /**
      * {@link ExecutorService} that performs installation.
      */
@@ -135,6 +145,10 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
         configure(new UpdateCenterConfiguration());
     }
 
+    public Api getApi() {
+        return new Api(this);
+    }
+
     /**
      * Configures update center to get plugins/updates from alternate servers,
      * and optionally using alternate strategies for downloading, installing
@@ -155,6 +169,7 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
      * @return
      *      can be empty but never null. Oldest entries first.
      */
+    @Exported
     public List<UpdateCenterJob> getJobs() {
         synchronized (jobs) {
             return new ArrayList<UpdateCenterJob>(jobs);
@@ -197,6 +212,7 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
      * @return
      *      can be empty but never null.
      */
+    @Exported
     public PersistedList<UpdateSite> getSites() {
         return sites;
     }
@@ -276,8 +292,8 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
     /**
      * Schedules a Jenkins upgrade.
      */
+    @RequirePOST
     public void doUpgrade(StaplerResponse rsp) throws IOException, ServletException {
-        requirePOST();
         Jenkins.getInstance().checkPermission(Jenkins.ADMINISTER);
         HudsonUpgradeJob job = new HudsonUpgradeJob(getCoreSource(), Jenkins.getAuthentication());
         if(!Lifecycle.get().canRewriteHudsonWar()) {
@@ -362,8 +378,8 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
     /**
      * Performs hudson downgrade.
      */
+    @RequirePOST
     public void doDowngrade(StaplerResponse rsp) throws IOException, ServletException {
-        requirePOST();
         Jenkins.getInstance().checkPermission(Jenkins.ADMINISTER);
         if(!isDowngradable()) {
             sendError("Jenkins downgrade is not possible, probably backup does not exist");
@@ -616,37 +632,41 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
          * @see DownloadJob
          */
         public File download(DownloadJob job, URL src) throws IOException {
-            URLConnection con = connect(job,src);
-            int total = con.getContentLength();
-            CountingInputStream in = new CountingInputStream(con.getInputStream());
-            byte[] buf = new byte[8192];
-            int len;
-
-            File dst = job.getDestination();
-            File tmp = new File(dst.getPath()+".tmp");
-            OutputStream out = new FileOutputStream(tmp);
-
-            LOGGER.info("Downloading "+job.getName());
             try {
-                while((len=in.read(buf))>=0) {
-                    out.write(buf,0,len);
-                    job.status = job.new Installing(total==-1 ? -1 : in.getCount()*100/total);
+                URLConnection con = connect(job,src);
+                int total = con.getContentLength();
+                CountingInputStream in = new CountingInputStream(con.getInputStream());
+                byte[] buf = new byte[8192];
+                int len;
+
+                File dst = job.getDestination();
+                File tmp = new File(dst.getPath()+".tmp");
+                OutputStream out = new FileOutputStream(tmp);
+
+                LOGGER.info("Downloading "+job.getName());
+                try {
+                    while((len=in.read(buf))>=0) {
+                        out.write(buf,0,len);
+                        job.status = job.new Installing(total==-1 ? -1 : in.getCount()*100/total);
+                    }
+                } catch (IOException e) {
+                    throw new IOException2("Failed to load "+src+" to "+tmp,e);
                 }
+
+                in.close();
+                out.close();
+
+                if (total!=-1 && total!=tmp.length()) {
+                    // don't know exactly how this happens, but report like
+                    // http://www.ashlux.com/wordpress/2009/08/14/hudson-and-the-sonar-plugin-fail-maveninstallation-nosuchmethoderror/
+                    // indicates that this kind of inconsistency can happen. So let's be defensive
+                    throw new IOException("Inconsistent file length: expected "+total+" but only got "+tmp.length());
+                }
+
+                return tmp;
             } catch (IOException e) {
-                throw new IOException2("Failed to load "+src+" to "+tmp,e);
+                throw new IOException2("Failed to download from "+src,e);
             }
-
-            in.close();
-            out.close();
-
-            if (total!=-1 && total!=tmp.length()) {
-                // don't know exactly how this happens, but report like
-                // http://www.ashlux.com/wordpress/2009/08/14/hudson-and-the-sonar-plugin-fail-maveninstallation-nosuchmethoderror/
-                // indicates that this kind of inconsistency can happen. So let's be defensive
-                throw new IOException("Inconsistent file length: expected "+total+" but only got "+tmp.length());
-            }
-
-            return tmp;
         }
 
         /**
@@ -706,7 +726,7 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
          *      Absolute URL that ends with '/'.
          */
         public String getUpdateCenterUrl() {
-            return "http://updates.jenkins-ci.org/";
+            return UPDATE_CENTER_URL;
         }
 
         /**
@@ -737,16 +757,26 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
      *
      * This object will have the <tt>row.jelly</tt> which renders the job on UI.
      */
+    @ExportedBean
     public abstract class UpdateCenterJob implements Runnable {
         /**
          * Which {@link UpdateSite} does this belong to?
          */
         public final UpdateSite site;
 
+        /**
+         * If this job fails, set to the error.
+         */
+        protected Throwable error;
+
         protected UpdateCenterJob(UpdateSite site) {
             this.site = site;
         }
 
+        public Api getApi() {
+            return new Api(this);
+        }
+
         /**
          * @deprecated as of 1.326
          *      Use {@link #submit()} instead.
@@ -755,6 +785,11 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
             submit();
         }
 
+        @Exported
+        public String getType() {
+            return getClass().getSimpleName();
+        }
+
         /**
          * Schedules this job for an execution
          * @return
@@ -765,6 +800,11 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
             jobs.add(this);
             return installerService.submit(this,this);
         }
+
+        @Exported
+        public Throwable getError() {
+            return error;
+        }
     }
 
     /**
@@ -774,11 +814,13 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
         /**
          * Unique ID that identifies this job.
          */
+        @Exported
         public final int id = iota.incrementAndGet();
                
          /**
          * Immutable state of this job.
          */
+         @Exported(inline=true)
         public volatile RestartJenkinsJobStatus status = new Pending();
         
         /**
@@ -803,21 +845,25 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
             status = new Running();
             try {
                 Jenkins.getInstance().safeRestart();
-            }
-            catch (RestartNotSupportedException exception) {
+            } catch (RestartNotSupportedException exception) {
                 // ignore if restart is not allowed
                 status = new Failure();
+                error = exception;
             }
         }
-        
+
+        @ExportedBean
         public abstract class RestartJenkinsJobStatus {
-            
+            @Exported
             public final int id = iota.incrementAndGet();
    
         }
         
         public class Pending extends RestartJenkinsJobStatus {
-            
+            @Exported
+            public String getType() {
+                return getClass().getSimpleName();
+            }
         }
         
         public class Running extends RestartJenkinsJobStatus {
@@ -867,8 +913,10 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
             } catch (UnknownHostException e) {
                 statuses.add(Messages.UpdateCenter_Status_UnknownHostException(e.getMessage()));
                 addStatus(e);
+                error = e;
             } catch (IOException e) {
                 statuses.add(Functions.printThrowable(e));
+                error = e;
             }
         }
 
@@ -890,10 +938,12 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
         /**
          * Unique ID that identifies this job.
          */
+        @Exported
         public final int id = iota.incrementAndGet();
         /**
          * Immutable object representing the current state of this job.
          */
+        @Exported(inline=true)
         public volatile InstallationStatus status = new Pending();
 
         /**
@@ -906,6 +956,7 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
          */
         protected abstract File getDestination();
 
+        @Exported
         public abstract String getName();
 
         /**
@@ -919,6 +970,7 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
         /**
          * Get the user that initiated this job
          */
+        @Exported
         public Authentication getUser() {
             return this.authentication;
         }
@@ -937,13 +989,17 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
                 LOGGER.info("Installation successful: "+getName());
                 status = new Success();
                 onSuccess();
+            } catch (InstallationStatus e) {
+                status = e;
+                if (status.isSuccess()) onSuccess();
             } catch (Throwable e) {
                 LOGGER.log(Level.SEVERE, "Failed to install "+getName(),e);
                 status = new Failure(e);
+                error = e;
             }
         }
 
-        protected void _run() throws IOException {
+        protected void _run() throws IOException, InstallationStatus {
             URL src = getURL();
 
             config.preValidate(this, src);
@@ -974,11 +1030,16 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
          * <p>
          * Instances of this class is immutable.
          */
-        public abstract class InstallationStatus {
+        public abstract class InstallationStatus extends Throwable {
             public final int id = iota.incrementAndGet();
+            @Exported
             public boolean isSuccess() {
                 return false;
             }
+            @Exported
+            public final String getType() {
+                return getClass().getSimpleName();
+            }
         }
 
         /**
@@ -991,11 +1052,28 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
                 this.problem = problem;
             }
 
-            public String getStackTrace() {
+            public String getProblemStackTrace() {
                 return Functions.printThrowable(problem);
             }
         }
 
+        /**
+         * Indicates that the installation was successful but a restart is needed.
+         *
+         * @see
+         */
+        public class SuccessButRequiresRestart extends Success {
+            private final Localizable message;
+
+            public SuccessButRequiresRestart(Localizable message) {
+                this.message = message;
+            }
+
+            public String getMessage() {
+                return message.toString();
+            }
+        }
+
         /**
          * Indicates that the plugin was successfully installed.
          */
@@ -1033,13 +1111,27 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
         /**
          * What plugin are we trying to install?
          */
+        @Exported
         public final Plugin plugin;
 
         private final PluginManager pm = Jenkins.getInstance().getPluginManager();
 
+        /**
+         * True to load the plugin into this Jenkins, false to wait until restart.
+         */
+        private final boolean dynamicLoad;
+
+        /**
+         * @deprecated as of 1.442
+         */
         public InstallationJob(Plugin plugin, UpdateSite site, Authentication auth) {
+            this(plugin,site,auth,false);
+        }
+
+        public InstallationJob(Plugin plugin, UpdateSite site, Authentication auth, boolean dynamicLoad) {
             super(site, auth);
             this.plugin = plugin;
+            this.dynamicLoad = dynamicLoad;
         }
 
         protected URL getURL() throws MalformedURLException {
@@ -1047,6 +1139,11 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
         }
 
         protected File getDestination() {
+            File baseDir = pm.rootDir;
+            return new File(baseDir, plugin.name + ".jpi");
+        }
+        
+        private File getLegacyDestination() {
             File baseDir = pm.rootDir;
             return new File(baseDir, plugin.name + ".hpi");
         }
@@ -1056,18 +1153,31 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
         }
 
         @Override
-        public void _run() throws IOException {
+        public void _run() throws IOException, InstallationStatus {
             super._run();
 
             // if this is a bundled plugin, make sure it won't get overwritten
             PluginWrapper pw = plugin.getInstalled();
-            if (pw!=null && pw.isBundled())
+            if (pw!=null && pw.isBundled()) {
+                SecurityContext oldContext = ACL.impersonate(ACL.SYSTEM);
                 try {
-                    SecurityContextHolder.getContext().setAuthentication(ACL.SYSTEM);
                     pw.doPin();
                 } finally {
-                    SecurityContextHolder.clearContext();
+                    SecurityContextHolder.setContext(oldContext);
                 }
+            }
+
+            if (dynamicLoad) {
+                try {
+                    pm.dynamicLoad(getDestination());
+                } catch (RestartRequiredException e) {
+                    throw new SuccessButRequiresRestart(e.message);
+                } catch (Exception e) {
+                    throw new IOException2("Failed to dynamically deploy this plugin",e);
+                }
+            } else {
+                throw new SuccessButRequiresRestart(Messages._UpdateCenter_DownloadButNotActivated());
+            }
         }
 
         protected void onSuccess() {
@@ -1078,6 +1188,29 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
         public String toString() {
             return super.toString()+"[plugin="+plugin.title+"]";
         }
+        
+        /**
+         * Called when the download is completed to overwrite
+         * the old file with the new file.
+         */
+        @Override
+        protected void replace(File dst, File src) throws IOException {
+        	File bak = Util.changeExtension(dst,".bak");
+        	
+            bak.delete();
+            final File legacy = getLegacyDestination();
+			if(legacy.exists()){
+            	legacy.renameTo(bak);
+            }else{
+            	dst.renameTo(bak);
+            }
+            legacy.delete();
+            dst.delete(); // any failure up to here is no big deal
+            
+            if(!src.renameTo(dst)) {
+                throw new IOException("Failed to rename "+src+" to "+dst);
+            }
+        }
     }
 
     /**
@@ -1102,7 +1235,11 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
 
         protected File getDestination() {
             File baseDir = pm.rootDir;
-            return new File(baseDir, plugin.name + ".hpi");
+            final File legacy = new File(baseDir, plugin.name + ".hpi");
+            if(legacy.exists()){
+            	return legacy;
+            }
+            return new File(baseDir, plugin.name + ".jpi");
         }
 
         protected File getBackup() {
@@ -1127,6 +1264,7 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
             } catch (Throwable e) {
                 LOGGER.log(Level.SEVERE, "Failed to downgrade "+getName(),e);
                 status = new Failure(e);
+                error = e;
             }
         }
 
@@ -1222,6 +1360,7 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
             } catch (Throwable e) {
                 LOGGER.log(Level.SEVERE, "Failed to downgrade "+getName(),e);
                 status = new Failure(e);
+                error = e;
             }
         }
 
@@ -1257,9 +1396,6 @@ public class UpdateCenter extends AbstractModelObject implements Saveable {
      */
     @Extension
     public static class PageDecoratorImpl extends PageDecorator {
-        public PageDecoratorImpl() {
-            super(PageDecoratorImpl.class);
-        }
     }
 
     /**
