@@ -23,13 +23,17 @@
  */
 package hudson.os.windows;
 
+import static hudson.Util.copyStreamAndClose;
+import static org.jvnet.hudson.wmi.Win32Service.Win32OwnProcess;
 import hudson.Extension;
 import hudson.Util;
 import hudson.lifecycle.WindowsSlaveInstaller;
+import hudson.model.AbstractDescribableImpl;
 import hudson.model.Computer;
 import hudson.model.Descriptor;
-import jenkins.model.Jenkins;
 import hudson.model.TaskListener;
+import hudson.os.windows.ManagedWindowsServiceAccount.AnotherUser;
+import hudson.os.windows.ManagedWindowsServiceAccount.LocalSystem;
 import hudson.remoting.Channel;
 import hudson.remoting.Channel.Listener;
 import hudson.remoting.SocketInputStream;
@@ -42,21 +46,6 @@ import hudson.tools.JDKInstaller.Platform;
 import hudson.util.IOUtils;
 import hudson.util.Secret;
 import hudson.util.jna.DotNet;
-import jcifs.smb.NtlmPasswordAuthentication;
-import jcifs.smb.SmbException;
-import jcifs.smb.SmbFile;
-import org.apache.commons.lang.StringUtils;
-import org.dom4j.Document;
-import org.dom4j.DocumentException;
-import org.dom4j.io.SAXReader;
-import org.jinterop.dcom.common.JIDefaultAuthInfoImpl;
-import org.jinterop.dcom.common.JIException;
-import org.jinterop.dcom.core.JISession;
-import org.jvnet.hudson.remcom.WindowsRemoteProcessLauncher;
-import org.jvnet.hudson.wmi.SWbemServices;
-import org.jvnet.hudson.wmi.WMI;
-import org.jvnet.hudson.wmi.Win32Service;
-import org.kohsuke.stapler.DataBoundConstructor;
 
 import java.io.BufferedInputStream;
 import java.io.BufferedOutputStream;
@@ -73,8 +62,23 @@ import java.net.UnknownHostException;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
-import static hudson.Util.copyStreamAndClose;
-import static org.jvnet.hudson.wmi.Win32Service.Win32OwnProcess;
+import jcifs.smb.NtlmPasswordAuthentication;
+import jcifs.smb.SmbException;
+import jcifs.smb.SmbFile;
+import jenkins.model.Jenkins;
+
+import org.apache.commons.lang.StringUtils;
+import org.dom4j.Document;
+import org.dom4j.DocumentException;
+import org.dom4j.io.SAXReader;
+import org.jinterop.dcom.common.JIDefaultAuthInfoImpl;
+import org.jinterop.dcom.common.JIException;
+import org.jinterop.dcom.core.JISession;
+import org.jvnet.hudson.remcom.WindowsRemoteProcessLauncher;
+import org.jvnet.hudson.wmi.SWbemServices;
+import org.jvnet.hudson.wmi.WMI;
+import org.jvnet.hudson.wmi.Win32Service;
+import org.kohsuke.stapler.DataBoundConstructor;
 
 /**
  * Windows slave installed/managed as a service entirely remotely
@@ -82,12 +86,57 @@ import static org.jvnet.hudson.wmi.Win32Service.Win32OwnProcess;
  * @author Kohsuke Kawaguchi
  */
 public class ManagedWindowsServiceLauncher extends ComputerLauncher {
+
     /**
      * "[DOMAIN\\]USERNAME" to follow the Windows convention.
      */
     public final String userName;
     
     public final Secret password;
+    
+    public final String vmargs;
+
+    /**
+     * @deprecated Use {@link #account}
+     */
+    public transient final AccountInfo logOn;
+
+    /**
+     * Specifies the account used to run the service.
+     */
+    private ManagedWindowsServiceAccount account;
+    
+    public static class LogOnOption {
+        public final String value;
+
+        public final AccountInfo logOn;
+
+        @DataBoundConstructor
+        public LogOnOption(String value, AccountInfo logOn) {
+            this.value = value;
+            this.logOn = logOn;
+        }
+    }
+
+    public static class AccountInfo extends AbstractDescribableImpl<AccountInfo> {
+        public final String userName;
+
+        public final Secret password;
+
+        @DataBoundConstructor
+        public AccountInfo(String userName, String password) {
+            this.userName = userName;
+            this.password = Secret.fromString(password);
+        }
+
+        @Extension
+        public static class DescriptorImpl extends Descriptor<AccountInfo> {
+            @Override
+            public String getDisplayName() {
+                return ""; // unused
+            }
+        }
+    }
 
     /**
      * Host name to connect to. For compatibility reasons, null if the same with the slave name.
@@ -98,12 +147,29 @@ public class ManagedWindowsServiceLauncher extends ComputerLauncher {
     public ManagedWindowsServiceLauncher(String userName, String password) {
         this (userName, password, null);
     }
+
+    public ManagedWindowsServiceLauncher(String userName, String password, String host) {
+        this(userName, password, host, null, null);
+    }
+
+    public ManagedWindowsServiceLauncher(String userName, String password, String host, AccountInfo account) {
+        this(userName,password,host,account==null ? new LocalSystem() : new AnotherUser(account.userName,account.password), null);
+    }
     
     @DataBoundConstructor
-    public ManagedWindowsServiceLauncher(String userName, String password, String host) {
+    public ManagedWindowsServiceLauncher(String userName, String password, String host, ManagedWindowsServiceAccount account, String vmargs) {
         this.userName = userName;
         this.password = Secret.fromString(password);
+        this.vmargs = Util.fixEmptyAndTrim(vmargs);
         this.host = Util.fixEmptyAndTrim(host);
+        this.account = account==null ? new LocalSystem() : account;
+        this.logOn = null;
+    }
+
+    public Object readResolve() {
+        if (logOn!=null)
+            account = new AnotherUser(logOn.userName,logOn.password);
+        return this;
     }
 
     private JIDefaultAuthInfoImpl createAuth() {
@@ -118,6 +184,15 @@ public class ManagedWindowsServiceLauncher extends ComputerLauncher {
         return new NtlmPasswordAuthentication(auth.getDomain(), auth.getUserName(), auth.getPassword());
     }
 
+    public ManagedWindowsServiceAccount getAccount() {
+        return account;
+    }
+
+    private AccountInfo getLogOn() {
+        if (account==null)  return null;
+        return account.getAccount(this);
+    }
+
     @Override
     public void launch(final SlaveComputer computer, final TaskListener listener) throws IOException, InterruptedException {
         try {
@@ -137,14 +212,7 @@ public class ManagedWindowsServiceLauncher extends ComputerLauncher {
 //                // continue anyway, just in case it's just ICMP that's getting filtered
 //            }
 
-            try {
-                Socket s = new Socket();
-                s.connect(new InetSocketAddress(host,135),5000);
-                s.close();
-            } catch (IOException e) {
-                logger.println("Failed to connect to port 135 of "+name+". Is Windows firewall blocking this port? Or did you disable DCOM service?");
-                // again, let it continue.
-            }
+            checkPort135Access(logger, name, host);
 
             JIDefaultAuthInfoImpl auth = createAuth();
             JISession session = JISession.createSession(auth);
@@ -230,26 +298,42 @@ public class ManagedWindowsServiceLauncher extends ComputerLauncher {
                 copySlaveJar(logger, remoteRoot);
 
                 // copy jenkins-slave.xml
-                logger.println(Messages.ManagedWindowsServiceLauncher_CopyingSlaveXml());
-                String xml = WindowsSlaveInstaller.generateSlaveXml(id,"javaw.exe","-tcp %BASE%\\port.txt");
-                copyStreamAndClose(new ByteArrayInputStream(xml.getBytes("UTF-8")), new SmbFile(remoteRoot,"jenkins-slave.xml").getOutputStream());
+                String xml = createAndCopyJenkinsSlaveXml(id, logger, remoteRoot);
 
                 // install it as a service
                 logger.println(Messages.ManagedWindowsServiceLauncher_RegisteringService());
                 Document dom = new SAXReader().read(new StringReader(xml));
                 Win32Service svc = services.Get("Win32_Service").cast(Win32Service.class);
-                int r = svc.Create(
+                int r;
+                AccountInfo logOn = getLogOn();
+                if (logOn == null) {
+                    r = svc.Create(
                         id,
                         dom.selectSingleNode("/service/name").getText()+" at "+path,
                         path+"\\jenkins-slave.exe",
                         Win32OwnProcess, 0, "Manual", true);
+                } else {
+                    r = svc.Create(
+                        id,
+                        dom.selectSingleNode("/service/name").getText()+" at "+path,
+                        path+"\\jenkins-slave.exe",
+                        Win32OwnProcess,
+                        0,
+                        "Manual",
+                        false, // When using a different user, it isn't possible to interact
+                        logOn.userName,
+                        Secret.toString(logOn.password),
+                        null, null, null);
+
+                }
                 if(r!=0) {
                     listener.error("Failed to create a service: "+svc.getErrorMessage(r));
                     return;
                 }
                 slaveService = services.getService(id);
             } else {
-                copySlaveJar(logger, remoteRoot);                
+                createAndCopyJenkinsSlaveXml(id, logger, remoteRoot);
+                copySlaveJar(logger, remoteRoot);
             }
 
             logger.println(Messages.ManagedWindowsServiceLauncher_StartingService());
@@ -295,10 +379,22 @@ public class ManagedWindowsServiceLauncher extends ComputerLauncher {
         }
     }
 
+    private void checkPort135Access(PrintStream logger, String name, InetAddress host) throws IOException {
+        Socket s = new Socket();
+        try {
+            s.connect(new InetSocketAddress(host,135),5000);
+        } catch (IOException e) {
+            logger.println("Failed to connect to port 135 of "+name+". Is Windows firewall blocking this port? Or did you disable DCOM service?");
+            // again, let it continue.
+        } finally {
+            s.close();
+        }
+    }
+
     /**
      * Determines the host name (or the IP address) to connect to.
      */
-    protected String determineHost(Computer c) throws IOException, InterruptedException {
+    protected String determineHost(Computer c) {
         // If host not provided, default to the slave name
         if (StringUtils.isBlank(host)) {
             return c.getName();
@@ -306,10 +402,17 @@ public class ManagedWindowsServiceLauncher extends ComputerLauncher {
             return host;
         }
     }
+    
+    private String createAndCopyJenkinsSlaveXml(String serviceId, PrintStream logger, SmbFile remoteRoot) throws IOException {
+        logger.println(Messages.ManagedWindowsServiceLauncher_CopyingSlaveXml());
+        String xml = WindowsSlaveInstaller.generateSlaveXml(serviceId,"javaw.exe",vmargs,"-tcp %BASE%\\port.txt");
+        copyStreamAndClose(new ByteArrayInputStream(xml.getBytes("UTF-8")), new SmbFile(remoteRoot,"jenkins-slave.xml").getOutputStream());
+        return xml;
+    }
 
     private void copySlaveJar(PrintStream logger, SmbFile remoteRoot) throws IOException {
         // copy slave.jar
-        logger.println("Copying slave.jar");
+        logger.println(Messages.ManagedWindowsServiceLauncher_CopyingSlaveJar());
         copyStreamAndClose(Jenkins.getInstance().getJnlpJars("slave.jar").getURL().openStream(), new SmbFile(remoteRoot,"slave.jar").getOutputStream());
     }
 
@@ -329,11 +432,14 @@ public class ManagedWindowsServiceLauncher extends ComputerLauncher {
             JIDefaultAuthInfoImpl auth = createAuth();
             JISession session = JISession.createSession(auth);
             session.setGlobalSocketTimeout(60000);
-            SWbemServices services = WMI.connect(session, computer.getName());
-            Win32Service slaveService = services.getService("jenkinsslave");
+            SWbemServices services = WMI.connect(session, determineHost(computer));
+            String id = WindowsSlaveInstaller.generateServiceId(computer.getNode().getRemoteFS());
+            Win32Service slaveService = services.getService(id);
             if(slaveService!=null) {
                 listener.getLogger().println(Messages.ManagedWindowsServiceLauncher_StoppingService());
                 slaveService.StopService();
+                listener.getLogger().println(Messages.ManagedWindowsServiceLauncher_UnregisteringService());
+                slaveService.Delete();
             }
             //destroy session to free the socket	
             JISession.destroySession(session);
@@ -341,6 +447,8 @@ public class ManagedWindowsServiceLauncher extends ComputerLauncher {
             e.printStackTrace(listener.error(e.getMessage()));
         } catch (JIException e) {
             e.printStackTrace(listener.error(e.getMessage()));
+        } catch (IOException e) {
+            e.printStackTrace(listener.error(e.getMessage()));
         }
     }
 
@@ -351,7 +459,9 @@ public class ManagedWindowsServiceLauncher extends ComputerLauncher {
         }
     }
 
+    private static final Logger JINTEROP_LOGGER = Logger.getLogger("org.jinterop");
+
     static {
-        Logger.getLogger("org.jinterop").setLevel(Level.WARNING);
+        JINTEROP_LOGGER.setLevel(Level.WARNING);
     }
 }
