@@ -30,10 +30,14 @@ import hudson.FilePath;
 import hudson.FilePath.FileCallable;
 import hudson.Launcher;
 import hudson.Util;
+import hudson.matrix.MatrixConfiguration;
 import hudson.model.AbstractBuild;
 import hudson.model.AbstractProject;
-import hudson.model.Build;
+import hudson.model.Action;
 import hudson.model.BuildListener;
+import hudson.model.DependecyDeclarer;
+import hudson.model.DependencyGraph;
+import hudson.model.DependencyGraph.Dependency;
 import hudson.model.Fingerprint;
 import hudson.model.Fingerprint.BuildPtr;
 import hudson.model.FingerprintMap;
@@ -41,10 +45,12 @@ import jenkins.model.Jenkins;
 import hudson.model.Result;
 import hudson.model.Run;
 import hudson.model.RunAction;
+import hudson.model.TaskListener;
 import hudson.remoting.VirtualChannel;
 import hudson.util.FormValidation;
 import hudson.util.IOException2;
 import hudson.util.PackedMap;
+import hudson.util.RunList;
 import net.sf.json.JSONObject;
 import org.apache.tools.ant.DirectoryScanner;
 import org.apache.tools.ant.types.FileSet;
@@ -59,9 +65,12 @@ import java.io.Serializable;
 import java.lang.ref.WeakReference;
 import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
+import java.util.ListIterator;
 import java.util.Map;
 import java.util.Map.Entry;
+import java.util.Set;
 import java.util.TreeMap;
 import java.util.logging.Level;
 import java.util.logging.Logger;
@@ -71,8 +80,9 @@ import java.util.logging.Logger;
  *
  * @author Kohsuke Kawaguchi
  */
-public class Fingerprinter extends Recorder implements Serializable {
-
+public class Fingerprinter extends Recorder implements Serializable, DependecyDeclarer {
+    public static boolean enableFingerprintsInDependencyGraph = Boolean.parseBoolean(System.getProperty(Fingerprinter.class.getName() + ".enableFingerprintsInDependencyGraph", "false"));
+    
     /**
      * Comma-separated list of files/directories to be fingerprinted.
      */
@@ -124,6 +134,9 @@ public class Fingerprinter extends Recorder implements Serializable {
 
             build.getActions().add(new FingerprintAction(build,record));
 
+            if (enableFingerprintsInDependencyGraph) {
+                Jenkins.getInstance().rebuildDependencyGraph();
+            }
         } catch (IOException e) {
             e.printStackTrace(listener.error(Messages.Fingerprinter_Failed()));
             build.setResult(Result.FAILURE);
@@ -137,6 +150,46 @@ public class Fingerprinter extends Recorder implements Serializable {
         return BuildStepMonitor.NONE;
     }
 
+    public void buildDependencyGraph(AbstractProject owner, DependencyGraph graph) {
+        if (enableFingerprintsInDependencyGraph) {
+            RunList builds = owner.getBuilds();
+            Set<String> seenUpstreamProjects = new HashSet<String>();
+
+            for ( ListIterator iter = builds.listIterator(); iter.hasNext(); ) {
+                Run build = (Run) iter.next();
+                List<FingerprintAction> fingerprints = build.getActions(FingerprintAction.class);
+                for (FingerprintAction action : fingerprints) {
+                    Map<AbstractProject,Integer> deps = action.getDependencies();
+                    for (AbstractProject key : deps.keySet()) {
+                        if (key == owner) {
+                            continue;   // Avoid self references
+                        }
+
+                        AbstractProject p = key;
+                        if (key instanceof MatrixConfiguration) {
+                            p = key.getRootProject();
+                        }
+
+                        if (seenUpstreamProjects.contains(p.getName())) {
+                            continue;
+                        }
+
+                        seenUpstreamProjects.add(p.getName());
+                        graph.addDependency(new Dependency(p, owner) {
+                            @Override
+                            public boolean shouldTriggerBuild(AbstractBuild build,
+                                                              TaskListener listener,
+                                                              List<Action> actions) {
+                                // Fingerprints should not trigger builds.
+                                return false;
+                            }
+                        });
+                    }
+                }
+            }
+        }
+    }
+
     private void record(AbstractBuild<?,?> build, BuildListener listener, Map<String,String> record, final String targets) throws IOException, InterruptedException {
         final class Record implements Serializable {
             final boolean produced;
@@ -239,6 +292,7 @@ public class Fingerprinter extends Recorder implements Serializable {
      * Action for displaying fingerprints.
      */
     public static final class FingerprintAction implements RunAction {
+        
         private final AbstractBuild build;
 
         /**
@@ -353,10 +407,20 @@ public class Fingerprinter extends Recorder implements Serializable {
         }
 
         /**
-         * Gets the dependency to other builds in a map.
-         * Returns build numbers instead of {@link Build}, since log records may be gone.
+         * Gets the dependency to other existing builds in a map.
          */
         public Map<AbstractProject,Integer> getDependencies() {
+            return getDependencies(false);
+        }
+        
+        /**
+         * Gets the dependency to other builds in a map.
+         *
+         * @param includeMissing true if the original build should be included in
+         *  the result, even if it doesn't exist
+         * @since 1.430
+         */
+        public Map<AbstractProject,Integer> getDependencies(boolean includeMissing) {
             Map<AbstractProject,Integer> r = new HashMap<AbstractProject,Integer>();
 
             for (Fingerprint fp : getFingerprints().values()) {
@@ -364,9 +428,11 @@ public class Fingerprinter extends Recorder implements Serializable {
                 if(bp==null)    continue;       // outside Hudson
                 if(bp.is(build))    continue;   // we are the owner
                 AbstractProject job = bp.getJob();
-                if (job==null)  continue;   // no longer exists
+                if (job==null)  continue;   // project no longer exists
                 if (job.getParent()==build.getParent())
                     continue;   // we are the parent of the build owner, that is almost like we are the owner 
+                if(job.getBuildByNumber(bp.getNumber())==null && !includeMissing)
+                    continue;               // build no longer exists
 
                 Integer existing = r.get(job);
                 if(existing!=null && existing>bp.getNumber())
