@@ -38,6 +38,9 @@ import javax.mail.Transport;
 import javax.mail.internet.InternetAddress;
 import javax.mail.internet.MimeMessage;
 import javax.mail.internet.AddressException;
+
+import org.apache.commons.lang.StringUtils;
+
 import java.io.File;
 import java.io.IOException;
 import java.util.*;
@@ -138,7 +141,7 @@ public class MailSender {
         do {
             b=b.getPreviousBuild();
             if(b==null) return null;
-        } while(b.getResult()==Result.ABORTED);
+        } while((b.getResult()==Result.ABORTED) || (b.getResult()==Result.NOT_BUILT));
         return b.getResult();
     }
 
@@ -305,11 +308,19 @@ public class MailSender {
         MimeMessage msg = new MimeMessage(Mailer.descriptor().createSession());
         // TODO: I'd like to put the URL to the page in here,
         // but how do I obtain that?
+        msg.addHeader("X-Jenkins-Job", build.getProject().getDisplayName());
+        msg.addHeader("X-Jenkins-Result", build.getResult().toString());
         msg.setContent("", "text/plain");
         msg.setFrom(new InternetAddress(Mailer.descriptor().getAdminAddress()));
         msg.setSentDate(new Date());
+        
+        String replyTo = Mailer.descriptor().getReplyToAddress();
+        if (StringUtils.isNotBlank(replyTo)) {
+            msg.setHeader("Reply-To", replyTo);
+        }
 
         Set<InternetAddress> rcp = new LinkedHashSet<InternetAddress>();
+        String defaultSuffix = Mailer.descriptor().getDefaultSuffix();
         StringTokenizer tokens = new StringTokenizer(recipients);
         while (tokens.hasMoreTokens()) {
             String address = tokens.nextToken();
@@ -324,10 +335,17 @@ public class MailSender {
                 includeCulpritsOf(up, build, listener, rcp);
             } else {
                 // ordinary address
+            	
+            	// if not a valid address (i.e. no '@'), then try adding suffix
+            	if (!address.contains("@") && defaultSuffix != null && defaultSuffix.contains("@")) {
+            		address += defaultSuffix;
+            	}
+            	
                 try {
                     rcp.add(new InternetAddress(address));
                 } catch (AddressException e) {
                     // report bad address, but try to send to other addresses
+                    listener.getLogger().println("Unable to send to address: " + address);
                     e.printStackTrace(listener.error(e.getMessage()));
                 }
             }
