@@ -172,7 +172,8 @@ public class RobustReflectionConverter implements Converter {
     }
 
     protected void marshallField(final MarshallingContext context, Object newObj, Field field) {
-        context.convertAnother(newObj);
+        Converter converter = mapper.getLocalConverter(field.getDeclaringClass(), field.getName());
+        context.convertAnother(newObj, converter);
     }
 
     public Object unmarshal(final HierarchicalStreamReader reader, final UnmarshallingContext context) {
@@ -287,7 +288,8 @@ public class RobustReflectionConverter implements Converter {
     }
 
     protected Object unmarshalField(final UnmarshallingContext context, final Object result, Class type, Field field) {
-        return context.convertAnother(result, type);
+        Converter converter = mapper.getLocalConverter(field.getDeclaringClass(), field.getName());
+        return context.convertAnother(result, type, converter);
     }
 
     private Map writeValueToImplicitCollection(UnmarshallingContext context, Object value, Map implicitCollections, Object result, String itemFieldName) {
