@@ -23,6 +23,9 @@
  */
 package hudson;
 
+import hudson.FilePath.TarCompression;
+import hudson.model.TaskListener;
+import hudson.remoting.LocalChannel;
 import hudson.remoting.VirtualChannel;
 import hudson.util.IOException2;
 import hudson.util.NullStream;
@@ -42,6 +45,8 @@ import java.util.concurrent.Executors;
 
 import org.apache.commons.io.FileUtils;
 import org.apache.commons.io.output.NullOutputStream;
+import org.apache.tools.ant.Project;
+import org.apache.tools.ant.taskdefs.Chmod;
 import org.jvnet.hudson.test.Bug;
 
 /**
@@ -297,4 +302,110 @@ public class FilePathTest extends ChannelTestCase {
         }
     }
 
+    public void testListWithDefaultExcludes() throws Exception {
+        File baseDir = Util.createTempDir();
+        try {
+            final Set<FilePath> expected = new HashSet<FilePath>();
+            expected.add(createFilePath(baseDir, "top", "sub", "backup~"));
+            expected.add(createFilePath(baseDir, "top", "CVS", "somefile,v"));
+            expected.add(createFilePath(baseDir, "top", ".git", "config"));
+            // none of the files are included by default (default includes true)
+            assertEquals(0, new FilePath(baseDir).list("**", "").length);
+            final FilePath[] result = new FilePath(baseDir).list("**", "", false);
+            assertEquals(expected, new HashSet<FilePath>(Arrays.asList(result)));
+        } finally {
+            Util.deleteRecursive(baseDir);
+        }
+    }
+
+    @Bug(11073)
+    public void testIsUnix() {
+        FilePath winPath = new FilePath(new LocalChannel(null),
+                " c:\\app\\hudson\\workspace\\3.8-jelly-db\\jdk/jdk1.6.0_21/label/sqlserver/profile/sqlserver\\acceptance-tests\\distribution.zip");
+        assertFalse(winPath.isUnix());
+
+        FilePath base = new FilePath(new LocalChannel(null),
+                "c:\\app\\hudson\\workspace\\3.8-jelly-db");
+        FilePath middle = new FilePath(base, "jdk/jdk1.6.0_21/label/sqlserver/profile/sqlserver");
+        FilePath full = new FilePath(middle, "acceptance-tests\\distribution.zip");
+        assertFalse(full.isUnix());
+        
+        
+        FilePath unixPath = new FilePath(new LocalChannel(null),
+                "/home/test");
+        assertTrue(unixPath.isUnix());
+    }
+    
+    /**
+     * Tests that permissions are kept when using {@link FilePath#copyToWithPermission(FilePath)}.
+     * Also tries to check that a problem with setting the last-modified date on Windows doesn't fail the whole copy
+     * - well at least when running this test on a Windows OS. See JENKINS-11073
+     */
+    public void testCopyToWithPermission() throws IOException, InterruptedException {
+        File tmp = Util.createTempDir();
+        try {
+            File child = new File(tmp,"child");
+            FilePath childP = new FilePath(child);
+            childP.touch(4711);
+            
+            Chmod chmodTask = new Chmod();
+            chmodTask.setProject(new Project());
+            chmodTask.setFile(child);
+            chmodTask.setPerm("0400");
+            chmodTask.execute();
+            
+            FilePath copy = new FilePath(british,tmp.getPath()).child("copy");
+            childP.copyToWithPermission(copy);
+            
+            assertEquals(childP.mode(),copy.mode());
+            if (!Functions.isWindows()) {
+                assertEquals(childP.lastModified(),copy.lastModified());
+            }
+            
+            // JENKINS-11073:
+            // Windows seems to have random failures when setting the timestamp on newly generated
+            // files. So test that:
+            for (int i=0; i<100; i++) {
+                copy = new FilePath(british,tmp.getPath()).child("copy"+i);
+                childP.copyToWithPermission(copy);
+            }
+        } finally {
+            Util.deleteRecursive(tmp);
+        }
+    }
+
+    public void testSymlinkInTar() throws Exception {
+        if (Functions.isWindows())  return; // can't test on Windows
+
+        FilePath tmp = new FilePath(Util.createTempDir());
+        try {
+            FilePath in = tmp.child("in");
+            in.mkdirs();
+            in.child("c").touch(0);
+            in.child("b").symlinkTo("c", TaskListener.NULL);
+                        
+            FilePath tar = tmp.child("test.tar");
+            in.tar(tar.write(), "**/*");
+
+            FilePath dst = in.child("dst");
+            tar.untar(dst, TarCompression.NONE);
+
+            assertEquals("c",dst.child("b").readLink());
+        } finally {
+            tmp.deleteRecursive();
+        }
+    }
+
+    @Bug(13649)
+    public void testMultiSegmentRelativePaths() throws Exception {
+        FilePath winPath = new FilePath(new LocalChannel(null), "c:\\app\\jenkins\\workspace");
+        FilePath nixPath = new FilePath(new LocalChannel(null), "/opt/jenkins/workspace");
+
+        assertEquals("c:\\app\\jenkins\\workspace\\foo\\bar\\manchu", new FilePath(winPath, "foo/bar/manchu").getRemote());
+        assertEquals("c:\\app\\jenkins\\workspace\\foo\\bar\\manchu", new FilePath(winPath, "foo\\bar/manchu").getRemote());
+        assertEquals("c:\\app\\jenkins\\workspace\\foo\\bar\\manchu", new FilePath(winPath, "foo\\bar\\manchu").getRemote());
+        assertEquals("/opt/jenkins/workspace/foo/bar/manchu", new FilePath(nixPath, "foo\\bar\\manchu").getRemote());
+        assertEquals("/opt/jenkins/workspace/foo/bar/manchu", new FilePath(nixPath, "foo/bar\\manchu").getRemote());
+        assertEquals("/opt/jenkins/workspace/foo/bar/manchu", new FilePath(nixPath, "foo/bar/manchu").getRemote());
+    }
 }
