@@ -41,6 +41,7 @@ import java.util.logging.Logger;
 
 import org.apache.maven.plugin.descriptor.MojoDescriptor;
 import org.apache.maven.plugin.descriptor.PluginDescriptor;
+import org.codehaus.plexus.classworlds.realm.ClassRealm;
 import org.kohsuke.stapler.Stapler;
 
 /**
@@ -53,6 +54,7 @@ import org.kohsuke.stapler.Stapler;
  * @author Kohsuke Kawaguchi
  */
 public final class ExecutedMojo implements Serializable {
+    private static final long serialVersionUID = -3048316415397586490L;
     /**
      * Plugin group ID.
      */
@@ -83,7 +85,7 @@ public final class ExecutedMojo implements Serializable {
      */
     public final String digest;
 
-    public ExecutedMojo(MojoInfo mojo, long duration) throws IOException, InterruptedException {
+    public ExecutedMojo(MojoInfo mojo, long duration) {
         this.groupId = mojo.pluginName.groupId;
         this.artifactId = mojo.pluginName.artifactId;
         this.version = mojo.pluginName.version;
@@ -96,12 +98,17 @@ public final class ExecutedMojo implements Serializable {
         PluginDescriptor pd = md.getPluginDescriptor();
         try {
             Class<?> clazz = getMojoClass( md, pd );
-            digest = Util.getDigestOf(new FileInputStream(Which.jarFile(clazz)));
+            if (clazz!=null) {
+                digest = Util.getDigestOf(new FileInputStream(Which.jarFile(clazz)));    
+            } else {
+                LOGGER.log(Level.WARNING, "Failed to getClass for "+md.getImplementation());    
+            }
+            
         } catch (IllegalArgumentException e) {
             LOGGER.log(Level.WARNING, "Failed to locate jar for "+md.getImplementation(),e);
         } catch (ClassNotFoundException e) {
             // perhaps the plugin has failed to load.
-        } catch (FileNotFoundException e) {
+        } catch (IOException e) {
             // Maybe mojo was loaded from a classes dir instead of from a jar (JENKINS-5044)
             LOGGER.log(Level.WARNING, "Failed to caculate digest for "+md.getImplementation(),e);
         }
@@ -110,7 +117,8 @@ public final class ExecutedMojo implements Serializable {
     
     private Class<?> getMojoClass(MojoDescriptor md, PluginDescriptor pd) throws ClassNotFoundException {
         try {
-            return pd.getClassRealm().loadClass( md.getImplementation() );
+            ClassRealm classRealm = pd.getClassRealm();
+            return classRealm == null ? null : classRealm.loadClass( md.getImplementation() );
         } catch (NoSuchMethodError e) {
             // maybe we are in maven2 build ClassRealm package has changed
             return getMojoClassForMaven2( md, pd );
@@ -124,6 +132,10 @@ public final class ExecutedMojo implements Serializable {
         org.codehaus.classworlds.ClassRealm cl = 
             (org.codehaus.classworlds.ClassRealm) ReflectionUtils.invokeMethod( method, pd );
         
+        if (cl==null)
+        {
+            return null;
+        }
         Class<?> clazz = cl.loadClass( md.getImplementation() );
         return clazz;
        
@@ -149,7 +161,7 @@ public final class ExecutedMojo implements Serializable {
      *
      * TODO: better if XStream has a declarative way of marking fields as "target for intern".
      */
-    ExecutedMojo readResolve() {
+    protected Object readResolve() {
         return new ExecutedMojo(intern(groupId),intern(artifactId),intern(version),intern(goal),intern(executionId),duration,intern(digest));
     }
 
@@ -175,12 +187,16 @@ public final class ExecutedMojo implements Serializable {
             return Stapler.getCurrentRequest().getContextPath()+m.getUrl();
         if(groupId.equals("org.apache.maven.plugins"))
             return "http://maven.apache.org/plugins/"+artifactId+'/';
+        if (groupId.equals("org.codehaus.mojo"))
+            return "http://mojo.codehaus.org/"+artifactId+'/';
         return null;
     }
 
     public String getGoalLink(Cache c) {
         if(groupId.equals("org.apache.maven.plugins"))
             return "http://maven.apache.org/plugins/"+artifactId+'/'+goal+"-mojo.html";
+        if (groupId.equals("org.codehaus.mojo"))
+            return "http://mojo.codehaus.org/"+artifactId+'/'+goal+"-mojo.html";
         return null;
     }
 
