@@ -24,13 +24,21 @@
  */
 package hudson.maven;
 
-import static hudson.Util.fixEmpty;
+import static hudson.Util.*;
 import static hudson.model.ItemGroupMixIn.loadChildren;
 import hudson.CopyOnWrite;
+import hudson.EnvVars;
 import hudson.Extension;
+import hudson.ExtensionList;
 import hudson.FilePath;
+import hudson.Functions;
 import hudson.Indenter;
 import hudson.Util;
+import hudson.maven.local_repo.DefaultLocalRepositoryLocator;
+import hudson.maven.local_repo.LocalRepositoryLocator;
+import hudson.maven.local_repo.PerJobLocalRepositoryLocator;
+import hudson.maven.settings.SettingConfig;
+import hudson.maven.settings.SettingsProviderUtils;
 import hudson.model.AbstractProject;
 import hudson.model.Action;
 import hudson.model.BuildableItemWithBuildWrappers;
@@ -38,15 +46,16 @@ import hudson.model.DependencyGraph;
 import hudson.model.Descriptor;
 import hudson.model.Descriptor.FormException;
 import hudson.model.Executor;
-import jenkins.model.Jenkins;
 import hudson.model.Item;
 import hudson.model.ItemGroup;
 import hudson.model.Job;
 import hudson.model.Queue;
 import hudson.model.Queue.Task;
 import hudson.model.ResourceActivity;
+import hudson.model.Result;
 import hudson.model.SCMedItem;
 import hudson.model.Saveable;
+import hudson.model.TaskListener;
 import hudson.model.TopLevelItem;
 import hudson.search.CollectionSearchIndex;
 import hudson.search.SearchIndexBuilder;
@@ -54,12 +63,13 @@ import hudson.tasks.BuildStep;
 import hudson.tasks.BuildStepDescriptor;
 import hudson.tasks.BuildWrapper;
 import hudson.tasks.BuildWrappers;
+import hudson.tasks.Builder;
 import hudson.tasks.Fingerprinter;
-import hudson.tasks.JavadocArchiver;
 import hudson.tasks.Mailer;
 import hudson.tasks.Maven;
 import hudson.tasks.Maven.MavenInstallation;
 import hudson.tasks.Publisher;
+import hudson.tasks.JavadocArchiver;
 import hudson.tasks.junit.JUnitResultArchiver;
 import hudson.util.CopyOnWriteMap;
 import hudson.util.DescribableList;
@@ -82,6 +92,7 @@ import java.util.Stack;
 
 import javax.servlet.ServletException;
 
+import jenkins.model.Jenkins;
 import net.sf.json.JSONObject;
 
 import org.apache.commons.lang.math.NumberUtils;
@@ -104,6 +115,7 @@ import org.kohsuke.stapler.export.Exported;
  * @author Kohsuke Kawaguchi
  */
 public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenModuleSetBuild> implements TopLevelItem, ItemGroup<MavenModule>, SCMedItem, Saveable, BuildableItemWithBuildWrappers {
+	
     /**
      * All {@link MavenModule}s, keyed by their {@link MavenModule#getModuleName()} module name}s.
      */
@@ -172,8 +184,28 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
      * from multiple Maven process, so this helps there too.
      *
      * @since 1.223
+     * @deprecated as of 1.448
+     *      Subsumed by {@link #localRepository}. false maps to {@link DefaultLocalRepositoryLocator},
+     *      and true maps to {@link PerJobLocalRepositoryLocator}
      */
-    private boolean usePrivateRepository = false;
+    private transient Boolean usePrivateRepository;
+
+    /**
+     * Encapsulates where to run the local repository.
+     *
+     * If null, inherited from the global configuration.
+     * 
+     * @since 1.448
+     */
+    private LocalRepositoryLocator localRepository = null;
+    
+   /**
+    * If true, the build will send a failure e-mail for each failing maven module.
+    * Defaults to <code>true</code> to simulate old behavior. 
+    * <p>
+    * see JENKINS-5695. 
+    */
+    private Boolean perModuleEmail = Boolean.TRUE;
 
     /**
      * If true, do not automatically schedule a build when one of the project dependencies is built.
@@ -212,6 +244,21 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
      */
     private boolean runHeadless = false;
 
+    /**
+     * @since 1.426
+     */
+    private String settingConfigId;
+
+    /**
+     * @since 1.426
+     */
+    private String globalSettingConfigId;
+
+    /**
+     * used temporary during maven build to store file path
+     * @since 1.426
+     */
+    protected transient String globalSettingConfigPath;
     /**
      * Reporters configured at {@link MavenModuleSet} level. Applies to all {@link MavenModule} builds.
      */
@@ -232,6 +279,18 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
     private DescribableList<BuildWrapper,Descriptor<BuildWrapper>> buildWrappers =
         new DescribableList<BuildWrapper, Descriptor<BuildWrapper>>(this);
 
+	/**
+     * List of active {@link Builder}s configured for this project.
+     */
+    private DescribableList<Builder,Descriptor<Builder>> prebuilders =
+            new DescribableList<Builder,Descriptor<Builder>>(this);
+    
+    private DescribableList<Builder,Descriptor<Builder>> postbuilders =
+            new DescribableList<Builder,Descriptor<Builder>>(this);
+	
+    private Result runPostStepsIfResult;
+    
+   
     /**
      * @deprecated
      *      Use {@link #MavenModuleSet(ItemGroup, String)}
@@ -244,6 +303,39 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
         super(parent,name);
     }
 
+    /**
+     * Builders that are run before the main Maven execution.
+     *
+     * @since 1.433
+     */
+	public DescribableList<Builder,Descriptor<Builder>> getPrebuilders() {
+        return prebuilders;
+    }
+	
+    /**
+     * Builders that are run after the main Maven execution.
+     *
+     * @since 1.433
+     */
+	public DescribableList<Builder,Descriptor<Builder>> getPostbuilders() {
+        return postbuilders;
+    }
+	
+	/**
+     * {@link #postbuilders} are run if the result is better or equal to this threshold.
+     *
+     * @return
+     *      never null
+     * @since 1.433
+	 */
+	public Result getRunPostStepsIfResult() {
+		return Functions.defaulted(runPostStepsIfResult,Result.FAILURE);
+	}
+
+    public void setRunPostStepsIfResult(Result v) {
+        this.runPostStepsIfResult = Functions.defaulted(v,Result.FAILURE);
+    }
+
     public String getUrlChildPrefix() {
         // seemingly redundant "./" is used to make sure that ':' is not interpreted as the scheme identifier
         return ".";
@@ -361,10 +453,18 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
         return aggregatorStyleBuild;
     }
 
+    /**
+     * @deprecated as of 1.448
+     *      Use {@link #getLocalRepository()}
+     */
     public boolean usesPrivateRepository() {
-        return usePrivateRepository;
+        return !(getLocalRepository() instanceof DefaultLocalRepositoryLocator);
     }
 
+    public boolean isPerModuleEmail() {
+        return perModuleEmail;
+    }
+    
     public boolean ignoreUpstremChanges() {
         return ignoreUpstremChanges;
     }
@@ -385,8 +485,31 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
         this.aggregatorStyleBuild = aggregatorStyleBuild;
     }
 
+    /**
+     * @deprecated as of 1.448.
+     *      Use {@link #setLocalRepository(LocalRepositoryLocator)} instead
+     */
     public void setUsePrivateRepository(boolean usePrivateRepository) {
-        this.usePrivateRepository = usePrivateRepository;
+        setLocalRepository(usePrivateRepository?new PerJobLocalRepositoryLocator() : new DefaultLocalRepositoryLocator());
+    }
+
+    /**
+     * @return 
+     *      never null
+     */
+    public LocalRepositoryLocator getLocalRepository() {
+        return localRepository!=null ? localRepository : getDescriptor().getLocalRepository();
+    }
+
+    /**
+     * Undefaulted locally configured value with taking inheritance from the global configuration into account.
+     */
+    public LocalRepositoryLocator getExplicitLocalRepository() {
+        return localRepository;
+    }
+
+    public void setLocalRepository(LocalRepositoryLocator localRepository) {
+        this.localRepository = localRepository;
     }
 
     public void setIgnoreUpstremChanges(boolean ignoreUpstremChanges) {
@@ -420,8 +543,40 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
 
     public int getMavenValidationLevel() {
         return mavenValidationLevel;
-    }    
-    
+    }
+
+    /**
+     * @since 1.426
+     * @return
+     */
+    public String getSettingConfigId() {
+        return settingConfigId;
+    }
+
+    /**
+     * @since 1.426
+     * @param settingConfigId
+     */
+    public void setSettingConfigId( String settingConfigId ) {
+        this.settingConfigId = settingConfigId;
+    }
+
+    /**
+     * @since 1.426
+     * @return
+     */
+    public String getGlobalSettingConfigId() {
+        return globalSettingConfigId;
+    }
+
+    /**
+     * @since 1.426
+     * @param globalSettingConfigId
+     */
+    public void setGlobalSettingConfigId( String globalSettingConfigId ) {
+        this.globalSettingConfigId = globalSettingConfigId;
+    }
+
     /**
      * List of active {@link MavenReporter}s that should be applied to all module builds.
      */
@@ -541,16 +696,36 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
             this.sortedActiveModules = getDisabledModules(false);
         }
 
-        if(reporters==null)
+        if(reporters==null){
             reporters = new DescribableList<MavenReporter, Descriptor<MavenReporter>>(this);
+        }
         reporters.setOwner(this);
-        if(publishers==null)
+        if(publishers==null){
             publishers = new DescribableList<Publisher,Descriptor<Publisher>>(this);
+        }
         publishers.setOwner(this);
-        if(buildWrappers==null)
+        if(buildWrappers==null){
             buildWrappers = new DescribableList<BuildWrapper, Descriptor<BuildWrapper>>(this);
+        }
         buildWrappers.setOwner(this);
-
+        if(prebuilders==null){
+        	prebuilders = new DescribableList<Builder,Descriptor<Builder>>(this);
+        }
+        prebuilders.setOwner(this);
+        if(postbuilders==null){
+        	postbuilders = new DescribableList<Builder,Descriptor<Builder>>(this);
+        }
+        postbuilders.setOwner(this);
+        
+        if(perModuleEmail == null){
+            perModuleEmail = Boolean.TRUE;
+        }
+        
+        if (Boolean.TRUE.equals(usePrivateRepository)) {
+            this.localRepository = new PerJobLocalRepositoryLocator();
+            usePrivateRepository = null;
+        }
+        
         updateTransientActions();
     }
 
@@ -606,6 +781,8 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
         
         publishers.buildDependencyGraph(this,graph);
         buildWrappers.buildDependencyGraph(this,graph);
+        prebuilders.buildDependencyGraph(this,graph);
+        postbuilders.buildDependencyGraph(this,graph);
     }
 
     public MavenModule getRootModule() {
@@ -623,7 +800,9 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
 
         activities.addAll(super.getResourceActivities());
         activities.addAll(Util.filter(publishers,ResourceActivity.class));
-        activities.addAll(Util.filter(buildWrappers,ResourceActivity.class));
+        activities.addAll(Util.filter(buildWrappers, ResourceActivity.class));
+        activities.addAll(Util.filter(prebuilders,ResourceActivity.class));
+        activities.addAll(Util.filter(postbuilders,ResourceActivity.class));
 
         return activities;
     }
@@ -738,6 +917,11 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
      * to be used to launch Maven process.
      *
      * If mavenOpts is null or empty, we'll return the globally-defined MAVEN_OPTS.
+     *
+     * <p>
+     * This method returns a configured value as-is, which can include variabl references.
+     * At runtime, use {@link AbstractMavenBuild#getMavenOpts(TaskListener, EnvVars)} to obtain
+     * a fully resolved value.
      */
     public String getMavenOpts() {
         if ((mavenOpts!=null) && (mavenOpts.trim().length()>0)) { 
@@ -754,6 +938,22 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
         }
     }
 
+    /**
+     * @since 1.426
+     * @return
+     */
+    public List<SettingConfig> getAllMavenSettingsConfigs() {
+        return SettingsProviderUtils.getAllMavenSettingsConfigs();
+    }
+    
+    /**
+     * @since 1.426
+     * @return
+     */
+    public List<SettingConfig> getAllGlobalMavenSettingsConfigs() {
+        return SettingsProviderUtils.getAllGlobalMavenSettingsConfigs();
+    }
+
     /**
      * Set mavenOpts.
      */
@@ -829,7 +1029,11 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
         mavenOpts = Util.fixEmpty(req.getParameter("mavenOpts").trim());
         mavenName = req.getParameter("maven_version");
         aggregatorStyleBuild = !req.hasParameter("maven.perModuleBuild");
-        usePrivateRepository = req.hasParameter("maven.usePrivateRepository");
+        if (json.optBoolean("usePrivateRepository"))
+            localRepository = req.bindJSON(LocalRepositoryLocator.class,json.getJSONObject("explicitLocalRepository"));
+        else
+            localRepository = null;
+        perModuleEmail = req.hasParameter("maven.perModuleEmail");
         ignoreUpstremChanges = !json.has("triggerByDependency");
         runHeadless = req.hasParameter("maven.runHeadless");
         incrementalBuild = req.hasParameter("maven.incrementalBuild");
@@ -838,8 +1042,14 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
         processPlugins = req.hasParameter( "maven.processPlugins" );
         mavenValidationLevel = NumberUtils.toInt( req.getParameter( "maven.validationLevel" ), -1 );
         reporters.rebuild(req,json,MavenReporters.getConfigurableList());
-        publishers.rebuild(req,json,BuildStepDescriptor.filter(Publisher.all(),this.getClass()));
+        publishers.rebuildHetero(req, json, Publisher.all(), "publisher");
         buildWrappers.rebuild(req,json,BuildWrappers.getFor(this));
+        settingConfigId = req.getParameter( "maven.mavenSettingsConfigId" );
+        globalSettingConfigId = req.getParameter( "maven.mavenGlobalSettingConfigId" );
+
+        runPostStepsIfResult = Result.fromString(req.getParameter( "post-steps.runIfResult"));
+        prebuilders.rebuildHetero(req,json, Builder.all(), "prebuilder");
+        postbuilders.rebuildHetero(req,json, Builder.all(), "postbuilder");
     }
 
     /**
@@ -905,6 +1115,11 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
          */
         private Map<String, Integer> mavenValidationLevels = new LinkedHashMap<String, Integer>();
 
+        /**
+         * @since 1.448
+         */
+        private LocalRepositoryLocator localRepository = new DefaultLocalRepositoryLocator();
+
         public DescriptorImpl() {
             super();
             load();
@@ -925,6 +1140,18 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
             save();
         }
 
+        /**
+         * @return never null.
+         */
+        public LocalRepositoryLocator getLocalRepository() {
+            return localRepository!=null ? localRepository : new DefaultLocalRepositoryLocator();
+        }
+
+        public void setLocalRepository(LocalRepositoryLocator localRepository) {
+            this.localRepository = localRepository;
+            save();
+        }
+
         public String getDisplayName() {
             return Messages.MavenModuleSet_DiplayName();
         }
@@ -948,6 +1175,7 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
         @Override
         public boolean configure( StaplerRequest req, JSONObject o ) {
             globalMavenOpts = Util.fixEmptyAndTrim(o.getString("globalMavenOpts"));
+            localRepository = req.bindJSON(LocalRepositoryLocator.class,o.getJSONObject("localRepository"));
             save();
 
             return true;
@@ -964,13 +1192,5 @@ public class MavenModuleSet extends AbstractMavenProject<MavenModuleSet,MavenMod
             Mailer.class,           // for historical reasons, Maven uses MavenMailer
             JUnitResultArchiver.class // done by SurefireArchiver
         ));
-
-
     }
-
-
-
-
-
-
 }
