@@ -28,21 +28,35 @@ import hudson.FilePath;
 import hudson.Launcher;
 import hudson.Util;
 import hudson.maven.reporters.MavenAbstractArtifactRecord;
+import hudson.maven.reporters.MavenArtifactRecord;
+import hudson.maven.settings.SettingConfig;
+import hudson.maven.settings.SettingsProviderUtils;
 import hudson.model.AbstractBuild;
 import hudson.model.AbstractProject;
 import hudson.model.BuildListener;
-import jenkins.model.Jenkins;
 import hudson.model.Node;
 import hudson.model.Result;
 import hudson.model.TaskListener;
-import hudson.maven.reporters.MavenArtifactRecord;
 import hudson.remoting.Callable;
 import hudson.tasks.BuildStepDescriptor;
 import hudson.tasks.BuildStepMonitor;
 import hudson.tasks.Maven.MavenInstallation;
 import hudson.tasks.Publisher;
 import hudson.tasks.Recorder;
+
+import java.io.File;
+import java.io.IOException;
+import java.util.ArrayList;
+import java.util.Calendar;
+import java.util.Collections;
+import java.util.List;
+import java.util.Map.Entry;
+import java.util.Properties;
+
+import jenkins.model.Jenkins;
 import net.sf.json.JSONObject;
+
+import org.apache.commons.io.FileUtils;
 import org.apache.commons.lang.StringUtils;
 import org.apache.maven.artifact.Artifact;
 import org.apache.maven.artifact.deployer.ArtifactDeploymentException;
@@ -52,16 +66,12 @@ import org.apache.maven.artifact.repository.ArtifactRepositoryFactory;
 import org.apache.maven.artifact.repository.ArtifactRepositoryPolicy;
 import org.apache.maven.artifact.repository.Authentication;
 import org.apache.maven.artifact.repository.layout.ArtifactRepositoryLayout;
+import org.apache.maven.cli.BatchModeMavenTransferListener;
 import org.apache.maven.repository.Proxy;
 import org.codehaus.plexus.component.repository.exception.ComponentLookupException;
 import org.kohsuke.stapler.DataBoundConstructor;
 import org.kohsuke.stapler.StaplerRequest;
 
-import java.io.File;
-import java.io.IOException;
-import java.util.*;
-import java.util.Map.Entry;
-
 /**
  * {@link Publisher} for {@link MavenModuleSetBuild} to deploy artifacts
  * after a build is fully succeeded. 
@@ -80,30 +90,48 @@ public class RedeployPublisher extends Recorder {
     public final String url;
     public final boolean uniqueVersion;
     public final boolean evenIfUnstable;
+    public final String releaseEnvVar;
 
     /**
      * For backward compatibility
      */
     @Deprecated
     public RedeployPublisher(String id, String url, boolean uniqueVersion) {
-    	this(id, url, uniqueVersion, false);
+    	this(id, url, uniqueVersion, false, null);
     }
     
     /**
      * @since 1.347
      */
-    @DataBoundConstructor
+    @Deprecated
     public RedeployPublisher(String id, String url, boolean uniqueVersion, boolean evenIfUnstable) {
+        this(id, url, uniqueVersion, evenIfUnstable, null);
+    }
+    
+    @DataBoundConstructor
+    public RedeployPublisher(String id, String url, boolean uniqueVersion, boolean evenIfUnstable, String releaseEnvVar) {
         this.id = id;
         this.url = Util.fixEmptyAndTrim(url);
         this.uniqueVersion = uniqueVersion;
         this.evenIfUnstable = evenIfUnstable;
+        this.releaseEnvVar = Util.fixEmptyAndTrim(releaseEnvVar);
     }
 
     public boolean perform(AbstractBuild<?, ?> build, Launcher launcher, BuildListener listener) throws InterruptedException, IOException {
         if (build.getResult().isWorseThan(getTreshold()))
             return true;    // build failed. Don't publish
 
+        /**
+         * Check if we should skip or not
+         */
+        if (releaseEnvVar != null) {
+        	String envVarValue = build.getEnvironment(listener).get(releaseEnvVar);
+        	if ("true".equals(envVarValue)) { // null or false are ignored
+        		listener.getLogger().println("[INFO] Skipping deploying artifact as release build is in progress.");
+        		return true; // skip the deploy
+        	}
+        }
+        
         List<MavenAbstractArtifactRecord> mavenAbstractArtifactRecords = getActions(build, listener);
         if (mavenAbstractArtifactRecords == null || mavenAbstractArtifactRecords.isEmpty()) {
             listener.getLogger().println("[ERROR] No artifacts are recorded. Is this a Maven project?");
@@ -175,22 +203,20 @@ public class RedeployPublisher extends Recorder {
      */
     private MavenEmbedder createEmbedder(TaskListener listener, AbstractBuild<?,?> build) throws MavenEmbedderException, IOException, InterruptedException {
         MavenInstallation m=null;
-        File settingsLoc = null;
+        File settingsLoc = null, remoteGlobalSettingsFromConfig = null;
         String profiles = null;
         Properties systemProperties = null;
         String privateRepository = null;
+        FilePath remoteSettingsFromConfig = null;
         
         File tmpSettings = File.createTempFile( "jenkins", "temp-settings.xml" );
         try {
             AbstractProject project = build.getProject();
             
-            // don't care here as it's executed in the master
-            //if (project instanceof ProjectWithMaven) {
-            //    m = ((ProjectWithMaven) project).inferMavenInstallation().forNode(Hudson.getInstance(),listener);
-            //}
             if (project instanceof MavenModuleSet) {
-                profiles = ((MavenModuleSet) project).getProfiles();
-                systemProperties = ((MavenModuleSet) project).getMavenProperties();
+                MavenModuleSet mavenModuleSet = ((MavenModuleSet) project);
+                profiles = mavenModuleSet.getProfiles();
+                systemProperties = mavenModuleSet.getMavenProperties();
                 
                 // olamy see  
                 // we have to take about the settings use for the project
@@ -200,16 +226,50 @@ public class RedeployPublisher extends Recorder {
                 // if not we must get ~/.m2/settings.xml then $M2_HOME/conf/settings.xml
                 
                 // TODO check if the remoteSettings has a localRepository configured and disabled it
-                
-                String altSettingsPath = ((MavenModuleSet) project).getAlternateSettings();
-                
+
+                String settingsConfigId = mavenModuleSet.getSettingConfigId();
+                String altSettingsPath = null;
+
+                if (!StringUtils.isBlank(settingsConfigId)) {
+                    SettingConfig config = SettingsProviderUtils.findSettings(settingsConfigId);
+                    if (config == null) {
+                        listener.getLogger().println(
+                            " your Apache Maven build is setup to use a config with id " + settingsConfigId
+                                + " but cannot find the config" );
+                    } else {
+                        listener.getLogger().println( "redeploy publisher using settings config with name " + config.name );
+                        if (config.content != null ) {
+                            remoteSettingsFromConfig = SettingsProviderUtils.copyConfigContentToFilePath( config, build.getWorkspace() );
+                            altSettingsPath = remoteSettingsFromConfig.getRemote();
+                        }
+                    }
+                }
+
+                if (mavenModuleSet.getAlternateSettings() != null ) {
+                    altSettingsPath = mavenModuleSet.getAlternateSettings();
+                }
+
+                String globalSettingsConfigId = mavenModuleSet.getGlobalSettingConfigId();
+                if (!StringUtils.isBlank(globalSettingsConfigId)) {
+                    SettingConfig config = SettingsProviderUtils.findSettings(globalSettingsConfigId);
+                    if (config == null) {
+                        listener.getLogger().println(
+                            " your Apache Maven build is setup to use a global settings config with id "
+                                + globalSettingsConfigId + " but cannot find the config" );
+                    } else {
+                        listener.getLogger().println( "redeploy publisher using global settings config with name " + config.name );
+                        if (config.content != null ) {
+                            remoteGlobalSettingsFromConfig = SettingsProviderUtils.copyConfigContentToFile( config );
+                        }
+                    }
+                }
                 Node buildNode = build.getBuiltOn();
                 
                 if(buildNode == null) {
                     // assume that build was made on master
                     buildNode = Jenkins.getInstance();
                 }
-                
+
                 if (StringUtils.isBlank( altSettingsPath ) ) {
                     // get userHome from the node where job has been executed
                     String remoteUserHome = build.getWorkspace().act( new GetUserHome() );
@@ -233,19 +293,35 @@ public class RedeployPublisher extends Recorder {
                 settingsLoc = tmpSettings;
                 
             }
-            
-            return MavenUtil.createEmbedder(new MavenEmbedderRequest(listener,
+
+            MavenEmbedderRequest mavenEmbedderRequest = new MavenEmbedderRequest(listener,
                                   m!=null?m.getHomeDir():null,
                                   profiles,
                                   systemProperties,
                                   privateRepository,
-                                  settingsLoc ));
+                                  settingsLoc );
+
+            if (remoteGlobalSettingsFromConfig != null) {
+                mavenEmbedderRequest.setGlobalSettings( remoteGlobalSettingsFromConfig );
+            }
+
+            mavenEmbedderRequest.setTransferListener(new BatchModeMavenTransferListener(listener.getLogger()));
+
+            return MavenUtil.createEmbedder(mavenEmbedderRequest);
         } finally {
-            tmpSettings.delete();
+            if (tmpSettings != null) {
+                tmpSettings.delete();
+            }
+            if (remoteSettingsFromConfig != null) {
+                remoteSettingsFromConfig.delete();
+            }
+            FileUtils.deleteQuietly(remoteGlobalSettingsFromConfig);
         }
     }
     
     private static final class GetUserHome implements Callable<String,IOException> {
+        private static final long serialVersionUID = -8755705771716056636L;
+
         public String call() throws IOException {
             return System.getProperty("user.home");
         }
@@ -253,20 +329,23 @@ public class RedeployPublisher extends Recorder {
     
     
     /**
-     * Obtains the {@link MavenAbstractArtifactRecord} that we'll work on.
+     * Obtains the {@link MavenModuleSetBuild} that we'll work on, or null.
      * <p>
      * This allows promoted-builds plugin to reuse the code for delayed deployment. 
      */
-    protected MavenAbstractArtifactRecord getAction(AbstractBuild<?, ?> build) {
-        return build.getAction(MavenAbstractArtifactRecord.class);
+    protected MavenModuleSetBuild getMavenBuild(AbstractBuild<?, ?> build) {
+        return (build instanceof MavenModuleSetBuild)
+            ? (MavenModuleSetBuild) build
+            : null;
     }
     
     protected List<MavenAbstractArtifactRecord> getActions(AbstractBuild<?, ?> build, BuildListener listener) {
         List<MavenAbstractArtifactRecord> actions = new ArrayList<MavenAbstractArtifactRecord>();
-        if (!(build instanceof MavenModuleSetBuild)) {
+        MavenModuleSetBuild mavenBuild = getMavenBuild(build);
+        if (mavenBuild == null) {
             return actions;
         }
-        for (Entry<MavenModule, MavenBuild> e : ((MavenModuleSetBuild)build).getModuleLastBuilds().entrySet()) {
+        for (Entry<MavenModule, MavenBuild> e : mavenBuild.getModuleLastBuilds().entrySet()) {
             MavenAbstractArtifactRecord a = e.getValue().getAction( MavenAbstractArtifactRecord.class );
             if (a == null) {
                 listener.getLogger().println("No artifacts are recorded for module" + e.getKey().getName() + ". Is this a Maven project?");
@@ -324,7 +403,7 @@ public class RedeployPublisher extends Recorder {
     
     //---------------------------------------------
     
-    
+    @SuppressWarnings("deprecation") // as we're restricted to Maven 2.x API here, but compile against Maven 3.x we cannot avoid deprecations
     public static class WrappedArtifactRepository implements ArtifactRepository {
         private ArtifactRepository artifactRepository;
         private boolean uniqueVersion;
